using System;
using EnvDTE;
using System.IO;
using Extensibility;
using System.Diagnostics;
using System.Windows.Forms;
using System.Reflection;
using Microsoft.Office.Core;
using System.Runtime.InteropServices;
using System.Collections;

using DCLJobView;
using DCLJobView.TreeModel;
using DCLJobView.Events;

#region Read me for Add-in installation and setup information.
	// When run, the Add-in wizard prepared the registry for the Add-in.
	// At a later time, if the Add-in becomes unavailable for reasons such as:
	//   1) You moved this project to a computer other than which is was originally created on.
	//   2) You chose 'Yes' when presented with a message asking if you wish to remove the Add-in.
	//   3) Registry corruption.
	// you will need to re-register the Add-in by building the MyAddin21Setup project 
	// by right clicking the project in the Solution Explorer, then choosing install.
#endregion

namespace DCLAddin
{
	/// <summary>
	///		The object for implementing an Add-in.
	/// </summary>
	/// <seealso class='IDTExtensibility2' />
	[GuidAttribute("2C37088C-F420-4DCA-B3B4-4C54C4E63DB5"), ProgId("DCLAddin.Connect")]
	public class Connect : Object, Extensibility.IDTExtensibility2, IDTCommandTarget
	{
		// strings for Named Commands
		private const string PROG_ID                     = "DCLAddin.Connect";

		private const string SHORT_COMMAND_EXEC_JOB      = "DCLAddInExecJob";
		private const string SHORT_COMMAND_SHOW_JOB_VIEW = "DCLAddInShowJobView";
		private static readonly string SHORT_COMMAND_OPTIONS       = "DCLAddInOptions";

		public static readonly string COMMAND_EXEC_JOB      = PROG_ID + "." + SHORT_COMMAND_EXEC_JOB;
		public static readonly string COMMAND_SHOW_JOB_VIEW = PROG_ID + "." + SHORT_COMMAND_SHOW_JOB_VIEW;
		public static readonly string COMMAND_OPTIONS       = PROG_ID + "." + SHORT_COMMAND_OPTIONS;

		private const string EXEC_JOB_CAPTION      = "Send DCL Job...";
		private const string SHOW_JOB_VIEW_CAPTION = "Show DCL Job View";
		private const string OPTIONS_CAPTION       = "DCL Options...";
		
		private _DTE applicationObject;
		private AddIn addInInstance;

		private CommandBar dclToolBar;

		private OutputWindow outputWindow;
		private OutputWindowPane outputWindowPane;
		private EnvDTE.TaskListEvents taskListEvents;
		private EnvDTE.WindowEvents windowsEvents;
		private Hashtable addedTaskItems;
		private Hashtable openCodeWindows;

		private Window toolWindow;
		private JobViewControl JobView;

		/// <summary>
		///		Implements the constructor for the Add-in object.
		///		Place your initialization code within this method.
		/// </summary>
		public Connect()
		{
			this.dclToolBar = null;
		}

		/// <summary>
		///      Implements the OnConnection method of the IDTExtensibility2 interface.
		///      Receives notification that the Add-in is being loaded.
		/// </summary>
		/// <param term='application'>
		///      Root object of the host application.
		/// </param>
		/// <param term='connectMode'>
		///      Describes how the Add-in is being loaded.
		/// </param>
		/// <param term='addInInst'>
		///      Object representing this Add-in.
		/// </param>
		/// <seealso class='IDTExtensibility2' />
		public void OnConnection(object application, Extensibility.ext_ConnectMode connectMode, object addInInst, ref System.Array custom)
		{
			this.openCodeWindows = new Hashtable();
			this.addedTaskItems = new Hashtable();

			applicationObject = (_DTE)application;
			addInInstance = (AddIn)addInInst;

			/* Startup      = visual studio started and loaded the addin
			 * AfterStartup = the addin was loaded when visual studio was already running
			 *                (typically when manually starting the addin through the Addin-Manager) */
			if(connectMode == Extensibility.ext_ConnectMode.ext_cm_Startup
				|| connectMode == Extensibility.ext_ConnectMode.ext_cm_AfterStartup) //ext_cm_UISetup
			{
				object []contextGUIDS = new object[] { };

				Commands commands = applicationObject.Commands;
				_CommandBars commandBars = applicationObject.CommandBars;

				// create pane in output view
				this.outputWindow = (OutputWindow)applicationObject.Windows.Item(Constants.vsWindowKindOutput).Object;
				this.outputWindowPane = outputWindow.OutputWindowPanes.Add("DCL Add-In Logs");

				// hook for task navigated event (we only want the compile errors)
				this.taskListEvents = (EnvDTE.TaskListEvents)applicationObject.Events.get_TaskListEvents("BuildCompile");
				this.taskListEvents.TaskNavigated += new _dispTaskListEvents_TaskNavigatedEventHandler(this.taskListEvents_TaskNavigated);

				// hook for window closing events
				this.windowsEvents = (EnvDTE.WindowEvents)applicationObject.Events.get_WindowEvents(null);
				this.windowsEvents.WindowClosing += new _dispWindowEvents_WindowClosingEventHandler(windowsEvents_WindowClosing);
				this.windowsEvents.WindowActivated += new _dispWindowEvents_WindowActivatedEventHandler(windowsEvents_WindowActivated);

				// find or create tool bar
				this.dclToolBar = getCommandBar("Distributed Control Lab");
				if (this.dclToolBar == null)
					this.dclToolBar = (CommandBar) commands.AddCommandBar("Distributed Control Lab",
										vsCommandBarType.vsCommandBarTypeToolbar, null, 1);

				// add named commands to visual studio
				// first check if the commands are already available
				Command executeJob = this.getCommand(COMMAND_EXEC_JOB); //commands.Item(COMMAND_EXEC_JOB, -1);
				Command showJobView = this.getCommand(COMMAND_SHOW_JOB_VIEW); //commands.Item(COMMAND_SHOW_JOB_VIEW, -1);
				Command showOptions = this.getCommand(COMMAND_OPTIONS); //commands.Item(COMMAND_OPTIONS, -1);

				try
				{
					// create new commands if they do not exist already
					if (executeJob == null)
					{
						executeJob =
							commands.AddNamedCommand(addInInstance,				// addin instance
							SHORT_COMMAND_EXEC_JOB,								// name
							EXEC_JOB_CAPTION,									// button text
							"Sends a job to the DCL to execute it.",	// tooltip
							false,												// MS Office button
							4712,												// Bitmap Index
							ref contextGUIDS,									// ref array of context UI GUIDs
							// command disabled flags value
							(int)vsCommandStatus.vsCommandStatusSupported+(int)vsCommandStatus.vsCommandStatusEnabled);
					}

					if (showJobView == null)
					{
						showJobView =
							commands.AddNamedCommand(addInInstance,				// addin instance
							SHORT_COMMAND_SHOW_JOB_VIEW,						// name
							SHOW_JOB_VIEW_CAPTION,								// button text
							"Show the DCL job view tool window.",		// tooltip
							false,												// MS Office button
							4713,												// Bitmap Index
							ref contextGUIDS,									// ref array of context UI GUIDs
							// command disabled flags value
							(int)vsCommandStatus.vsCommandStatusSupported+(int)vsCommandStatus.vsCommandStatusEnabled);
					}

					if (showOptions == null)
					{
						showOptions =
							commands.AddNamedCommand(addInInstance,						// addin instance
							SHORT_COMMAND_OPTIONS,								        // name
							OPTIONS_CAPTION,											// button text
							"Shows the options dialog for the DCL extension add-in.",	// tooltip
							false,														// MS Office button
							4711,														// Bitmap Index
							ref contextGUIDS,											// ref array of context UI GUIDs
							// command disabled flags value
							(int)vsCommandStatus.vsCommandStatusSupported+(int)vsCommandStatus.vsCommandStatusEnabled);
					}
				}
				catch(System.Exception e)
				{
					Trace.WriteLine(e.Message);
					LoggingManager.log("Exception happend during command install: " + e.Message, this.outputWindowPane);
				}
				
				// add controls to the default code window popup menu
				CommandBar commandBarCodeWindow = (CommandBar)commandBars["Code Window"];

				// first check if the controls are already available
				CommandBarControl executeJobCodeWindowControl = getCommandBarControl(EXEC_JOB_CAPTION, commandBarCodeWindow);
				CommandBarControl showJobViewCodeWindowControl = getCommandBarControl(SHOW_JOB_VIEW_CAPTION, commandBarCodeWindow);
				CommandBarControl showOptionsCodeWindowControl = getCommandBarControl(OPTIONS_CAPTION, commandBarCodeWindow);

				try
				{
					// create new controls if they do not exist already
					if (executeJobCodeWindowControl == null && executeJob != null)
						executeJobCodeWindowControl = executeJob.AddControl(commandBarCodeWindow, 1);

					if (showJobViewCodeWindowControl == null && showJobView != null)
						showJobViewCodeWindowControl = showJobView.AddControl(commandBarCodeWindow, 2);

					if (showOptionsCodeWindowControl == null && showOptions != null)
						showOptionsCodeWindowControl = showOptions.AddControl(commandBarCodeWindow, 3);
				}
				catch(System.Exception e)
				{
					Trace.WriteLine(e.Message);
					LoggingManager.log("Exception happend during control install on code window popup menu: " + e.Message, this.outputWindowPane);
				}

				// first check if the controls are already available
				CommandBarControl executeJobToolBarControl = this.getCommandBarControl(EXEC_JOB_CAPTION, dclToolBar);
				CommandBarControl showJobViewToolBarControl = this.getCommandBarControl(SHOW_JOB_VIEW_CAPTION, dclToolBar);
				CommandBarControl showOptionsToolBarControl = this.getCommandBarControl(OPTIONS_CAPTION, dclToolBar);

				try
				{
					// create new controls if they do not exist already
					if (executeJobToolBarControl == null && executeJob != null)
					{
						executeJobToolBarControl = executeJob.AddControl(dclToolBar, 1);
						((CommandBarButton) executeJobToolBarControl).Style = MsoButtonStyle.msoButtonIcon;
					}

					if (showJobViewToolBarControl == null && showJobView != null)
					{
						showJobViewToolBarControl = showJobView.AddControl(dclToolBar, 2);
						((CommandBarButton) showJobViewToolBarControl).Style = MsoButtonStyle.msoButtonIcon;
					}

					if (showOptionsToolBarControl == null && showOptions != null)
					{
						showOptionsToolBarControl = showOptions.AddControl(dclToolBar, 3);
						((CommandBarButton) showOptionsToolBarControl).Style = MsoButtonStyle.msoButtonIcon;
					}
				}
				catch(System.Exception e)
				{
					Trace.WriteLine(e.Message);
					LoggingManager.log("Exception happend during control install on toolbar: " + e.Message, this.outputWindowPane);
				}
			}

			/* When the addin is loaded in AfterStartup, the interface method OnStartupComplete
			 * is not called. We must do it manually to register our tool window. */
			if (connectMode == Extensibility.ext_ConnectMode.ext_cm_AfterStartup)
			{
				this.RegisterToolWindow();

				if (this.dclToolBar != null)
					this.dclToolBar.Visible = true;
			}
		}

		/// <summary>
		///     Finds a named command by its command name string.
		/// </summary>
		private Command getCommand(string commandName)
		{
			if (commandName == null || commandName.Length == 0)
				return null;

			System.Collections.IEnumerator enumerator = applicationObject.Commands.GetEnumerator();
				
			while(enumerator.MoveNext())
			{
				if(commandName.Equals(((Command)enumerator.Current).Name))
					return (Command)enumerator.Current;
			}
			return null;
		}

		/// <summary>
		///     Finds a command control by its caption in a specified command bar.
		/// </summary>
		private CommandBarControl getCommandBarControl(string caption, CommandBar commandBar)
		{
			if (commandBar == null || caption == null || caption.Length == 0)
				return null;

			System.Collections.IEnumerator controlsEnum = commandBar.Controls.GetEnumerator();

			while(controlsEnum.MoveNext())
			{
				if(caption.Equals(((CommandBarControl)controlsEnum.Current).Caption))
					return (CommandBarControl)controlsEnum.Current;
			}
			return null;
		}

		/// <summary>
		///     Finds a commandbar.
		/// </summary>
		private CommandBar getCommandBar(string name)
		{
			if (name == null || name.Length == 0)
				return null;
			_CommandBars commandBars = applicationObject.CommandBars;

			System.Collections.IEnumerator enumerator = commandBars.GetEnumerator();

			while(enumerator.MoveNext())
			{
				if(name.Equals(((CommandBar)enumerator.Current).Name))
					return (CommandBar)enumerator.Current;
			}
			return null;
		}

		/// <summary>
		///     Implements the OnDisconnection method of the IDTExtensibility2 interface.
		///     Receives notification that the Add-in is being unloaded.
		/// </summary>
		/// <param term='disconnectMode'>
		///      Describes how the Add-in is being unloaded.
		/// </param>
		/// <param term='custom'>
		///      Array of parameters that are host application specific.
		/// </param>
		/// <seealso class='IDTExtensibility2' />
		public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom)
		{
			if (this.openCodeWindows != null)
				this.openCodeWindows.Clear();

			if (this.addedTaskItems != null)
				this.addedTaskItems.Clear();

			if (this.taskListEvents != null)
				this.taskListEvents.TaskNavigated -= new _dispTaskListEvents_TaskNavigatedEventHandler(this.taskListEvents_TaskNavigated);

			if (this.windowsEvents != null)
				this.windowsEvents.WindowClosing -= new _dispWindowEvents_WindowClosingEventHandler(this.windowsEvents_WindowClosing);

			// Note: normal situated removing of the command bar and of the commands is done in the uninstaller

			// when the user actively closes our addin, it should remove itself
			if(disconnectMode == Extensibility.ext_DisconnectMode.ext_dm_UserClosed)
			{
				Commands commands = applicationObject.Commands;

				commands.RemoveCommandBar(dclToolBar);

				// find the commands defined by us and remove them
				System.Collections.IEnumerator enumerator = commands.GetEnumerator();
				
				while(enumerator.MoveNext())
				{
					if(COMMAND_EXEC_JOB.Equals(((Command)enumerator.Current).Name) ||
					   COMMAND_SHOW_JOB_VIEW.Equals(((Command)enumerator.Current).Name) ||
					   COMMAND_OPTIONS.Equals(((Command)enumerator.Current).Name))
					{
						((Command)enumerator.Current).Delete();
					}
				}
			}
		}

		/// <summary>
		///      Implements the OnAddInsUpdate method of the IDTExtensibility2 interface.
		///      Receives notification that the collection of Add-ins has changed.
		/// </summary>
		/// <param term='custom'>
		///      Array of parameters that are host application specific.
		/// </param>
		/// <seealso class='IDTExtensibility2' />
		public void OnAddInsUpdate(ref System.Array custom) {}

		/// <summary>
		///      Implements the OnStartupComplete method of the IDTExtensibility2 interface.
		///      Receives notification that the host application has completed loading.
		/// </summary>
		/// <param term='custom'>
		///      Array of parameters that are host application specific.
		/// </param>
		/// <seealso class='IDTExtensibility2' />
		public void OnStartupComplete(ref System.Array custom)
		{
			this.RegisterToolWindow();
		}

		private void RegisterToolWindow()
		{
			/* Run this code before any CreateToolWindow calls (this is a crazy hack
			 * to ensure that the position of the toolwindow is saved by vstudio 
			 * between calls to CreateToolWindow) */
			Type tp = typeof( System.Windows.Forms.Control );
			tp = tp.GetNestedType( "ActiveXImpl", BindingFlags.NonPublic );
			FieldInfo fi = tp.GetField( "globalActiveXCount", BindingFlags.Static
				| BindingFlags.NonPublic );
			fi.SetValue( null, Convert.ToInt32( fi.GetValue( null ) ) + 1 );

			object ToolWinObj = null; 

			try
			{
				// create tool window
				this.toolWindow = applicationObject.Windows.CreateToolWindow(
					this.addInInstance, 
					"DCLJobView.JobViewControl", 
					"DCL Job View",
					"{9AB4CE8C-CD6E-49b2-82E0-4D6F20E173AA}", 
					ref ToolWinObj);  
			}
			catch(System.NotImplementedException e)
			{
				// if NotImplementedException was thrown,
				// we only try to create the tool window again
				// infos about this hack can be found at:
				// http://groups.google.de/groups?q=Windows.CreateToolWindow+NotImplementedException&hl=de&lr=&ie=UTF-8&oe=UTF-8&selm=%23hrt9rjCDHA.1600%40TK2MSFTNGP10.phx.gbl&rnum=1
				this.toolWindow = applicationObject.Windows.CreateToolWindow(
					this.addInInstance, 
					"DCLJobView.JobViewControl", 
					"DCL Job View",
					"{9AB4CE8C-CD6E-49b2-82E0-4D6F20E173AA}", 
					ref ToolWinObj);

				Trace.WriteLine(e.Message);
			}

			this.JobView = (JobViewControl)ToolWinObj;

			this.JobView.LogMessage += new DCLJobView.LogMessageEventHandler(this.JobView_LogMessage);
//			this.JobView.ShowBinaryResultClicked += new DCLJobView.BinaryResultEventHandler(this.JobView_ShowBinaryResultClicked);
			this.JobView.SendJobClicked += new System.EventHandler(this.JobView_SendJobClicked);
			this.JobView.ShowStringResultClicked += new DCLJobView.StringResultEventHandler(this.JobView_ShowStringResultClicked);
			this.JobView.ShowWebResultClicked += new WebResultEventHandler(this.JobView_ShowWebResultClicked);
			this.JobView.ShowCompilerResultClicked +=new CompilerResultEventHandler(JobView_ShowCompilerResultClicked);

			this.JobView.loadConfiguration();
		}

		/// <summary>
		///      Implements the OnBeginShutdown method of the IDTExtensibility2 interface.
		///      Receives notification that the host application is being unloaded.
		/// </summary>
		/// <param term='custom'>
		///      Array of parameters that are host application specific.
		/// </param>
		/// <seealso class='IDTExtensibility2' />
		public void OnBeginShutdown(ref System.Array custom) 
		{
			this.JobView = null;
			GC.Collect();
		}
		
		/// <summary>
		///      Implements the QueryStatus method of the IDTCommandTarget interface.
		///      This is called when the command's availability is updated.
		/// </summary>
		/// <param term='commandName'>
		///		The name of the command to determine state for.
		/// </param>
		/// <param term='neededText'>
		///		Text that is needed for the command.
		/// </param>
		/// <param term='status'>
		///		The state of the command in the user interface.
		/// </param>
		/// <param term='commandText'>
		///		Text requested by the neededText parameter.
		/// </param>
		/// <seealso class='Exec' />
		public void QueryStatus(string commandName, EnvDTE.vsCommandStatusTextWanted neededText, ref EnvDTE.vsCommandStatus status, ref object commandText)
		{
			if(neededText == EnvDTE.vsCommandStatusTextWanted.vsCommandStatusTextWantedNone)
			{
				// check if its one of our commands and return an appropriate status
				if(commandName.Equals(COMMAND_EXEC_JOB) ||
				   commandName.Equals(COMMAND_SHOW_JOB_VIEW) ||
				   commandName.Equals(COMMAND_OPTIONS))
				{
					status = (vsCommandStatus)vsCommandStatus.vsCommandStatusSupported|vsCommandStatus.vsCommandStatusEnabled;
				}
			}
		}

		/// <summary>
		///      Implements the Exec method of the IDTCommandTarget interface.
		///      This is called when the command is invoked.
		/// </summary>
		/// <param term='commandName'>
		///		The name of the command to execute.
		/// </param>
		/// <param term='executeOption'>
		///		Describes how the command should be run.
		/// </param>
		/// <param term='varIn'>
		///		Parameters passed from the caller to the command handler.
		/// </param>
		/// <param term='varOut'>
		///		Parameters passed from the command handler to the caller.
		/// </param>
		/// <param term='handled'>
		///		Informs the caller if the command was handled or not.
		/// </param>
		/// <seealso class='Exec' />
		public void Exec(string commandName, EnvDTE.vsCommandExecOption executeOption, ref object varIn, ref object varOut, ref bool handled)
		{
			handled = false;
			if(executeOption == EnvDTE.vsCommandExecOption.vsCommandExecOptionDoDefault)
			{
				// execute job
				if(commandName == COMMAND_EXEC_JOB)
				{
					this.toolWindow.Visible = true;
					this.JobView.startUpdateProcess();
					this.JobView_SendJobClicked(this, null);

					handled = true;
					return;
				}

				// open job view in tool window
				if(commandName == COMMAND_SHOW_JOB_VIEW)
				{
					this.toolWindow.Visible = true;
					this.JobView.startUpdateProcess();

					handled = true;
					return;
				}

				// open options window
				if(commandName == COMMAND_OPTIONS)
				{
					this.JobView.showOptions();

					handled = true;
					return;
				}
			}
		}

		private void JobView_LogMessage(object sender, DCLJobView.Events.MessageEvent e)
		{
			LoggingManager.log(e.Message, this.outputWindowPane);
		}

		private void JobView_SendJobClicked(object sender, System.EventArgs e)
		{
			if (applicationObject.ActiveDocument != null)
			{
				//Get a handle to the job document.
				Document doc = applicationObject.ActiveDocument;

				Window win = doc.ActiveWindow;

				if (win != null)
				{
					TextDocument objDoc = (TextDocument) doc.Object("TextDocument");

					if(objDoc != null)
					{
						objDoc.Selection.SelectAll();

						string job = objDoc.Selection.Text;
						objDoc.Selection.StartOfDocument(true);

						string ticket;

						// if this method was called by Exec()
						if (sender == this)
						{
							ticket = this.JobView.sendJob(job, false);
						}
						// SendJobToolBarButtonClicked event from JobviewControl
						else
						{
							ticket = this.JobView.sendJob(job, true);
						}

						if (ticket != null)
						{
							this.openCodeWindows.Add(ticket, win);
						}	
					}
				}
			}
		}

		private void JobView_ShowWebResultClicked(object sender, DCLJobView.Events.WebResultEvent e)
		{
			// show job results in web browser
			this.applicationObject.ItemOperations.Navigate(e.Url, EnvDTE.vsNavigateOptions.vsNavigateOptionsDefault);
		}

		private void JobView_ShowBinaryResultClicked(object sender, DCLJobView.Events.BinaryResultEvent e)
		{
			// TODO: show binary result
			if(e.Type.Equals("Diagramm"))
			{
			}

			if(e.Type.Equals("Flash"))
			{
			}
		}

		private void JobView_ShowStringResultClicked(object sender, DCLJobView.Events.StringResultEvent e)
		{
			Window win = null;

			string fileName;
			if(e.Name.Equals("Code"))
			{
				fileName = "Job";

				// check for an existing window with this job
				win = (Window) this.openCodeWindows[e.Ticket];
			}
			else
			{
				fileName = e.Name;
			}

			// if no window with this jobs exists, we create a new one
			if (win == null)
			{
				fileName += " " + this.JobView.getExperimentType(e.Ticket).Name + " " + this.JobView.getJob(e.Ticket).UsageStart.ToString();
				fileName = fileName.Replace(":", ".");

				// TODO: always readonly?
				win = this.openNewTextDocument(fileName, e.Result, true);

				if (win != null && e.Name.Equals("Code"))
					this.openCodeWindows.Add(e.Ticket, win);
			}
			// if the window already exists, make it visible
			else
				win.Activate();
		}

		private Window openNewTextDocument(string fileName, string text, bool readOnly)
		{
			Window win = null;
			//Create a new text document.
			if (this.applicationObject.LocaleID == 1031) // german
				win = this.applicationObject.ItemOperations.NewFile("Allgemein\\Textdatei", fileName, Constants.vsViewKindCode);
			else if (this.applicationObject.LocaleID == 1033) // us-english
				win = this.applicationObject.ItemOperations.NewFile("General\\Text File", fileName, Constants.vsViewKindCode);
			else
				return null;

			//Get a handle to the new document.
			TextDocument objTextDoc = (TextDocument)win.Document.Object("TextDocument");
			
			//Create an EditPoint and add some text.
			EditPoint objEditPoint = (EditPoint)objTextDoc.StartPoint.CreateEditPoint();
			objEditPoint.Insert(text);

			win.Document.ReadOnly = readOnly;

			return win;
		}

		private void taskListEvents_TaskNavigated(TaskItem TaskItem, ref bool NavigateHandled)
		{
			/* a simple this.addedTaskItems.Contains(TaskItem) does not work always, it seems
			 * as if the object would change inside... */
			TaskItem foundItem = null;

			foreach(TaskItem addedTaskItem in this.addedTaskItems.Keys)
			{
				if (addedTaskItem.Category == TaskItem.Category
					&& addedTaskItem.Description == TaskItem.Description
					&& addedTaskItem.Line == TaskItem.Line
					&& addedTaskItem.FileName == TaskItem.FileName)
				{
					foundItem = addedTaskItem;
					break;
				}
			}

			if (foundItem != null)
			{
				CompilerOutputItem item = (CompilerOutputItem) addedTaskItems[foundItem];

				Window win = (Window) this.openCodeWindows[item.Ticket];

				if (win != null)
				{
					win.Activate();

					TextDocument objTextDoc = (TextDocument)win.Document.Object("TextDocument");
					TextSelection sel = objTextDoc.Selection;

					if (item.Line > objTextDoc.EndPoint.Line)
						sel.MoveToLineAndOffset(objTextDoc.EndPoint.Line, 0, false);
					else
						sel.MoveToLineAndOffset(item.Line, item.Column, false);

					NavigateHandled = true;
				}
				else
				{
					// retrieve the code
					string result = this.JobView.getStringResult(item.Ticket, "Code");

					string fileName = "Job " + this.JobView.getExperimentType(item.Ticket).Name + " " + this.JobView.getJob(item.Ticket).UsageStart.ToString();
					fileName = fileName.Replace(":", ".");

					// TODO: always readonly?
					win = this.openNewTextDocument(fileName, result, true);

					if (win != null)
					{
						this.openCodeWindows.Add(item.Ticket, win);

						TextDocument objTextDoc = (TextDocument)win.Document.Object("TextDocument");
						TextSelection sel = objTextDoc.Selection;

						if (item.Line > objTextDoc.EndPoint.Line)
							sel.MoveToLineAndOffset(objTextDoc.EndPoint.Line, 0, false);
						else
							sel.MoveToLineAndOffset(item.Line, item.Column, false);

						NavigateHandled = true;
					}
				}
			}
		}

		private void windowsEvents_WindowClosing(Window Window)
		{
			// remove the window from the list
			// (deleting a hashtable entry by its value is rather difficult in .NET...)
			if (this.openCodeWindows.ContainsValue(Window))
			{
				string ticketToDelete = null;

				foreach (string ticket in this.openCodeWindows.Keys)
				{
					if ((Window) this.openCodeWindows[ticket] == Window)
					{
						ticketToDelete = ticket;
						break;
					}
				}

				if (ticketToDelete != null)
				{
					this.openCodeWindows.Remove(ticketToDelete);
				}
			}
		}

		private void windowsEvents_WindowActivated(Window GotFocus, Window LostFocus)
		{
			if (this.openCodeWindows.ContainsValue(GotFocus))
			{
				foreach (string ticket in this.openCodeWindows.Keys)
				{
					if ((Window) this.openCodeWindows[ticket] == GotFocus)
					{
						Result result = this.JobView.getResult(ticket, "Code");

						if (result != null)
							this.JobView.Tree.SelectedNode = result;

						break;
					}
				}
			}
		}

		private void JobView_ShowCompilerResultClicked(object sender, CompilerResultEvent e)
		{
			TaskList taskList = (TaskList) applicationObject.Windows.Item(Constants.vsWindowKindTaskList).Object;

			// delete our previously added items for a fresh task list
			foreach (TaskItem item in this.addedTaskItems.Keys)
			{
				try
				{
					if (item != null)
						item.Delete();
				}
				catch (NullReferenceException)
				{
					// this exception happens inside Delete() when the userCanDelete parameter is set to false
				}
			}

			this.addedTaskItems.Clear();

			// add one task item for each compiler output message
			foreach (CompilerOutputItem item in e.CompilerMessages)
			{
				// build a nice description which gets all information from the item
				string description = item.Message;

				if (item.ExtraMessage != null)
					description += " - " + item.ExtraMessage;

				if (item.Column != 0)
					description += " (at column " + item.Column + ")";

				// errors will have a higher priority than warnings
				EnvDTE.vsTaskPriority priority;

				if (item.Kind == CompilerOutputItem.Type.Warning)
					priority = EnvDTE.vsTaskPriority.vsTaskPriorityMedium;
				else
					priority = EnvDTE.vsTaskPriority.vsTaskPriorityHigh;


				TaskItem taskItem = taskList.TaskItems.Add("BuildCompile", e.Ticket, description,
					priority, vsTaskIcon.vsTaskIconCompile,
					false, item.Filename, item.Line, true, true);

				this.addedTaskItems.Add(taskItem, item);
			}

			applicationObject.Windows.Item(Constants.vsWindowKindTaskList).Visible = true;
		}
	}
}
