using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace DCLJobView
{
	public class AdvancedOptionsForm : System.Windows.Forms.Form
	{
	
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button okbutton;
		private System.Windows.Forms.Button cancelbutton;
		private System.Windows.Forms.TabControl tabControl1;
		private System.Windows.Forms.NumericUpDown numericUpDown1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.TabPage tabPage1;
		private System.Windows.Forms.TabPage tabPage2;
		private System.Windows.Forms.TabPage tabPage3;
		private System.Windows.Forms.TabPage tabPage4;
		private System.Windows.Forms.Panel panel2;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		private System.Windows.Forms.ColumnHeader columnHeader2;
		private System.Windows.Forms.Button buttonAddCLO;
		private System.Windows.Forms.Button buttonRemoveCLO;
		private System.Windows.Forms.ColorDialog colorDialog1;
		private System.Windows.Forms.Panel panel3;
		private System.Windows.Forms.Button buttonAddRCC;
		private System.Windows.Forms.Button buttonRemoveRCC;

		private Configuration config;
		private System.Windows.Forms.ListBox listBoxPatterns;
		private System.Windows.Forms.Button buttonRemovePattern;
		private System.Windows.Forms.Button buttonAddPattern;
		private System.Windows.Forms.Button buttonEditPattern;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Button buttonMoveUp;
		private System.Windows.Forms.Button buttonMoveDown;
		private System.Windows.Forms.ColumnHeader columnHeader3;
		private System.Windows.Forms.ColumnHeader columnHeader4;
		private System.Windows.Forms.ListView listViewCLO;
		private System.Windows.Forms.ListView listViewRCC;
		private System.Windows.Forms.Button buttonEditCLO;
		private System.Windows.Forms.Button buttonEditRCC;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;


		private ArrayList errorPatterns;
		private Hashtable codeLinesOverhead;
		private System.Windows.Forms.Label label5;
		private Hashtable resultCodeColors;

		public AdvancedOptionsForm(Configuration config)
		{
			InitializeComponent();

			this.config = config;

			// polling page
			this.numericUpDown1.Value = this.config.RefreshInterval;

			// error pattern page
			this.listBoxPatterns_SelectedIndexChanged(this, null);
			this.errorPatterns = new ArrayList();

			foreach(CompilerErrorPattern cep in this.config.ErrorPatterns)
			{
				CompilerErrorPattern copy = new CompilerErrorPattern(cep);
				this.errorPatterns.Add(cep);
			}

			this.listBoxPatterns.DataSource = this.errorPatterns;
			this.listBoxPatterns.DisplayMember = "Name";

			// code lines overhead page
			this.listViewCLO_SelectedIndexChanged(this, null);
			this.codeLinesOverhead = new Hashtable();

			foreach(string experiment in this.config.CodeLinesOverhead.Keys)
			{
				this.codeLinesOverhead.Add(experiment, this.config.CodeLinesOverhead[experiment]);
				ListViewItem item = new ListViewItem(new string[] {experiment, System.Convert.ToString((int) this.config.CodeLinesOverhead[experiment])});
				this.listViewCLO.Items.Add(item);
			}

			// result code color page
			this.listViewRCC_SelectedIndexChanged(this, null);
			this.resultCodeColors = new Hashtable();

			foreach(string resultCode in this.config.ResultCodeColors.Keys)
			{
				Color color = (Color) this.config.ResultCodeColors[resultCode];
				this.resultCodeColors.Add(resultCode, color);

				ListViewItem item = new ListViewItem(new string[] {resultCode, color.Name});
				item.ForeColor = color;
				this.listViewRCC.Items.Add(item);
			}
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Vom Windows Form-Designer generierter Code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.cancelbutton = new System.Windows.Forms.Button();
			this.okbutton = new System.Windows.Forms.Button();
			this.tabControl1 = new System.Windows.Forms.TabControl();
			this.tabPage1 = new System.Windows.Forms.TabPage();
			this.numericUpDown1 = new System.Windows.Forms.NumericUpDown();
			this.label1 = new System.Windows.Forms.Label();
			this.tabPage2 = new System.Windows.Forms.TabPage();
			this.buttonMoveDown = new System.Windows.Forms.Button();
			this.buttonMoveUp = new System.Windows.Forms.Button();
			this.label2 = new System.Windows.Forms.Label();
			this.listBoxPatterns = new System.Windows.Forms.ListBox();
			this.buttonRemovePattern = new System.Windows.Forms.Button();
			this.buttonAddPattern = new System.Windows.Forms.Button();
			this.buttonEditPattern = new System.Windows.Forms.Button();
			this.tabPage3 = new System.Windows.Forms.TabPage();
			this.listViewCLO = new System.Windows.Forms.ListView();
			this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
			this.panel2 = new System.Windows.Forms.Panel();
			this.buttonEditCLO = new System.Windows.Forms.Button();
			this.buttonRemoveCLO = new System.Windows.Forms.Button();
			this.buttonAddCLO = new System.Windows.Forms.Button();
			this.tabPage4 = new System.Windows.Forms.TabPage();
			this.listViewRCC = new System.Windows.Forms.ListView();
			this.columnHeader3 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader4 = new System.Windows.Forms.ColumnHeader();
			this.panel3 = new System.Windows.Forms.Panel();
			this.buttonEditRCC = new System.Windows.Forms.Button();
			this.buttonRemoveRCC = new System.Windows.Forms.Button();
			this.buttonAddRCC = new System.Windows.Forms.Button();
			this.colorDialog1 = new System.Windows.Forms.ColorDialog();
			this.label3 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.tabControl1.SuspendLayout();
			this.tabPage1.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDown1)).BeginInit();
			this.tabPage2.SuspendLayout();
			this.tabPage3.SuspendLayout();
			this.panel2.SuspendLayout();
			this.tabPage4.SuspendLayout();
			this.panel3.SuspendLayout();
			this.SuspendLayout();
			// 
			// cancelbutton
			// 
			this.cancelbutton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.cancelbutton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.cancelbutton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.cancelbutton.Location = new System.Drawing.Point(256, 376);
			this.cancelbutton.Name = "cancelbutton";
			this.cancelbutton.Size = new System.Drawing.Size(80, 23);
			this.cancelbutton.TabIndex = 21;
			this.cancelbutton.Text = "Cancel";
			this.cancelbutton.Click += new System.EventHandler(this.cancelbutton_Click);
			// 
			// okbutton
			// 
			this.okbutton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.okbutton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.okbutton.Location = new System.Drawing.Point(160, 376);
			this.okbutton.Name = "okbutton";
			this.okbutton.Size = new System.Drawing.Size(80, 23);
			this.okbutton.TabIndex = 20;
			this.okbutton.Text = "OK";
			this.okbutton.Click += new System.EventHandler(this.okbutton_Click);
			// 
			// tabControl1
			// 
			this.tabControl1.Controls.Add(this.tabPage1);
			this.tabControl1.Controls.Add(this.tabPage2);
			this.tabControl1.Controls.Add(this.tabPage3);
			this.tabControl1.Controls.Add(this.tabPage4);
			this.tabControl1.Dock = System.Windows.Forms.DockStyle.Top;
			this.tabControl1.Location = new System.Drawing.Point(4, 4);
			this.tabControl1.Name = "tabControl1";
			this.tabControl1.SelectedIndex = 0;
			this.tabControl1.Size = new System.Drawing.Size(338, 360);
			this.tabControl1.TabIndex = 0;
			// 
			// tabPage1
			// 
			this.tabPage1.Controls.Add(this.numericUpDown1);
			this.tabPage1.Controls.Add(this.label1);
			this.tabPage1.DockPadding.All = 8;
			this.tabPage1.Location = new System.Drawing.Point(4, 22);
			this.tabPage1.Name = "tabPage1";
			this.tabPage1.Size = new System.Drawing.Size(330, 334);
			this.tabPage1.TabIndex = 0;
			this.tabPage1.Text = "Polling";
			// 
			// numericUpDown1
			// 
			this.numericUpDown1.Location = new System.Drawing.Point(152, 24);
			this.numericUpDown1.Maximum = new System.Decimal(new int[] {
																		   60,
																		   0,
																		   0,
																		   0});
			this.numericUpDown1.Name = "numericUpDown1";
			this.numericUpDown1.Size = new System.Drawing.Size(80, 20);
			this.numericUpDown1.TabIndex = 1;
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 28);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(136, 16);
			this.label1.TabIndex = 6;
			this.label1.Text = "Job View Refresh Interval:";
			// 
			// tabPage2
			// 
			this.tabPage2.Controls.Add(this.label5);
			this.tabPage2.Controls.Add(this.buttonMoveDown);
			this.tabPage2.Controls.Add(this.buttonMoveUp);
			this.tabPage2.Controls.Add(this.label2);
			this.tabPage2.Controls.Add(this.listBoxPatterns);
			this.tabPage2.Controls.Add(this.buttonRemovePattern);
			this.tabPage2.Controls.Add(this.buttonAddPattern);
			this.tabPage2.Controls.Add(this.buttonEditPattern);
			this.tabPage2.DockPadding.All = 8;
			this.tabPage2.Location = new System.Drawing.Point(4, 22);
			this.tabPage2.Name = "tabPage2";
			this.tabPage2.Size = new System.Drawing.Size(330, 334);
			this.tabPage2.TabIndex = 1;
			this.tabPage2.Text = "Compiler Errors";
			// 
			// buttonMoveDown
			// 
			this.buttonMoveDown.Location = new System.Drawing.Point(237, 264);
			this.buttonMoveDown.Name = "buttonMoveDown";
			this.buttonMoveDown.Size = new System.Drawing.Size(80, 23);
			this.buttonMoveDown.TabIndex = 7;
			this.buttonMoveDown.Text = "Move Down";
			this.buttonMoveDown.Click += new System.EventHandler(this.buttonMoveDown_Click);
			// 
			// buttonMoveUp
			// 
			this.buttonMoveUp.Location = new System.Drawing.Point(237, 224);
			this.buttonMoveUp.Name = "buttonMoveUp";
			this.buttonMoveUp.Size = new System.Drawing.Size(80, 23);
			this.buttonMoveUp.TabIndex = 6;
			this.buttonMoveUp.Text = "Move Up";
			this.buttonMoveUp.Click += new System.EventHandler(this.buttonMoveUp_Click);
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(13, 16);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(128, 16);
			this.label2.TabIndex = 16;
			this.label2.Text = "Compiler Error Patterns:";
			// 
			// listBoxPatterns
			// 
			this.listBoxPatterns.Location = new System.Drawing.Point(13, 40);
			this.listBoxPatterns.Name = "listBoxPatterns";
			this.listBoxPatterns.Size = new System.Drawing.Size(208, 251);
			this.listBoxPatterns.TabIndex = 2;
			this.listBoxPatterns.DoubleClick += new System.EventHandler(this.buttonEditPattern_Click);
			this.listBoxPatterns.SelectedIndexChanged += new System.EventHandler(this.listBoxPatterns_SelectedIndexChanged);
			// 
			// buttonRemovePattern
			// 
			this.buttonRemovePattern.Location = new System.Drawing.Point(237, 80);
			this.buttonRemovePattern.Name = "buttonRemovePattern";
			this.buttonRemovePattern.Size = new System.Drawing.Size(80, 23);
			this.buttonRemovePattern.TabIndex = 4;
			this.buttonRemovePattern.Text = "Remove";
			this.buttonRemovePattern.Click += new System.EventHandler(this.buttonRemovePattern_Click);
			// 
			// buttonAddPattern
			// 
			this.buttonAddPattern.Location = new System.Drawing.Point(237, 40);
			this.buttonAddPattern.Name = "buttonAddPattern";
			this.buttonAddPattern.Size = new System.Drawing.Size(80, 23);
			this.buttonAddPattern.TabIndex = 3;
			this.buttonAddPattern.Text = "Add...";
			this.buttonAddPattern.Click += new System.EventHandler(this.buttonAddPattern_Click);
			// 
			// buttonEditPattern
			// 
			this.buttonEditPattern.Location = new System.Drawing.Point(237, 120);
			this.buttonEditPattern.Name = "buttonEditPattern";
			this.buttonEditPattern.Size = new System.Drawing.Size(80, 23);
			this.buttonEditPattern.TabIndex = 5;
			this.buttonEditPattern.Text = "Edit...";
			this.buttonEditPattern.Click += new System.EventHandler(this.buttonEditPattern_Click);
			// 
			// tabPage3
			// 
			this.tabPage3.Controls.Add(this.listViewCLO);
			this.tabPage3.Controls.Add(this.panel2);
			this.tabPage3.DockPadding.Bottom = 8;
			this.tabPage3.DockPadding.Left = 8;
			this.tabPage3.DockPadding.Right = 8;
			this.tabPage3.DockPadding.Top = 14;
			this.tabPage3.Location = new System.Drawing.Point(4, 22);
			this.tabPage3.Name = "tabPage3";
			this.tabPage3.Size = new System.Drawing.Size(330, 334);
			this.tabPage3.TabIndex = 2;
			this.tabPage3.Text = "Code Lines Overhead";
			// 
			// listViewCLO
			// 
			this.listViewCLO.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						  this.columnHeader1,
																						  this.columnHeader2});
			this.listViewCLO.Dock = System.Windows.Forms.DockStyle.Fill;
			this.listViewCLO.FullRowSelect = true;
			this.listViewCLO.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listViewCLO.LabelEdit = true;
			this.listViewCLO.LabelWrap = false;
			this.listViewCLO.Location = new System.Drawing.Point(8, 14);
			this.listViewCLO.MultiSelect = false;
			this.listViewCLO.Name = "listViewCLO";
			this.listViewCLO.Size = new System.Drawing.Size(314, 248);
			this.listViewCLO.TabIndex = 8;
			this.listViewCLO.View = System.Windows.Forms.View.Details;
			this.listViewCLO.AfterLabelEdit += new System.Windows.Forms.LabelEditEventHandler(this.listViewCLO_AfterLabelEdit);
			this.listViewCLO.BeforeLabelEdit += new System.Windows.Forms.LabelEditEventHandler(this.listViewCLO_BeforeLabelEdit);
			this.listViewCLO.SelectedIndexChanged += new System.EventHandler(this.listViewCLO_SelectedIndexChanged);
			// 
			// columnHeader1
			// 
			this.columnHeader1.Text = "Experiment Name";
			this.columnHeader1.Width = 172;
			// 
			// columnHeader2
			// 
			this.columnHeader2.Text = "Additional Code Lines";
			this.columnHeader2.Width = 123;
			// 
			// panel2
			// 
			this.panel2.Controls.Add(this.label3);
			this.panel2.Controls.Add(this.buttonEditCLO);
			this.panel2.Controls.Add(this.buttonRemoveCLO);
			this.panel2.Controls.Add(this.buttonAddCLO);
			this.panel2.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panel2.Location = new System.Drawing.Point(8, 262);
			this.panel2.Name = "panel2";
			this.panel2.Size = new System.Drawing.Size(314, 64);
			this.panel2.TabIndex = 0;
			// 
			// buttonEditCLO
			// 
			this.buttonEditCLO.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.buttonEditCLO.Location = new System.Drawing.Point(184, 32);
			this.buttonEditCLO.Name = "buttonEditCLO";
			this.buttonEditCLO.TabIndex = 11;
			this.buttonEditCLO.Text = "Edit Line...";
			this.buttonEditCLO.Click += new System.EventHandler(this.buttonEditCLO_Click);
			// 
			// buttonRemoveCLO
			// 
			this.buttonRemoveCLO.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.buttonRemoveCLO.Location = new System.Drawing.Point(96, 32);
			this.buttonRemoveCLO.Name = "buttonRemoveCLO";
			this.buttonRemoveCLO.TabIndex = 10;
			this.buttonRemoveCLO.Text = "Remove";
			this.buttonRemoveCLO.Click += new System.EventHandler(this.buttonRemoveCLO_Click);
			// 
			// buttonAddCLO
			// 
			this.buttonAddCLO.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.buttonAddCLO.Location = new System.Drawing.Point(8, 32);
			this.buttonAddCLO.Name = "buttonAddCLO";
			this.buttonAddCLO.TabIndex = 9;
			this.buttonAddCLO.Text = "Add";
			this.buttonAddCLO.Click += new System.EventHandler(this.buttonAddCLO_Click);
			// 
			// tabPage4
			// 
			this.tabPage4.Controls.Add(this.listViewRCC);
			this.tabPage4.Controls.Add(this.panel3);
			this.tabPage4.DockPadding.Bottom = 8;
			this.tabPage4.DockPadding.Left = 8;
			this.tabPage4.DockPadding.Right = 8;
			this.tabPage4.DockPadding.Top = 14;
			this.tabPage4.Location = new System.Drawing.Point(4, 22);
			this.tabPage4.Name = "tabPage4";
			this.tabPage4.Size = new System.Drawing.Size(330, 334);
			this.tabPage4.TabIndex = 3;
			this.tabPage4.Text = "Result Codes";
			// 
			// listViewRCC
			// 
			this.listViewRCC.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						  this.columnHeader3,
																						  this.columnHeader4});
			this.listViewRCC.Dock = System.Windows.Forms.DockStyle.Fill;
			this.listViewRCC.FullRowSelect = true;
			this.listViewRCC.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listViewRCC.LabelEdit = true;
			this.listViewRCC.Location = new System.Drawing.Point(8, 14);
			this.listViewRCC.MultiSelect = false;
			this.listViewRCC.Name = "listViewRCC";
			this.listViewRCC.Size = new System.Drawing.Size(314, 248);
			this.listViewRCC.TabIndex = 11;
			this.listViewRCC.View = System.Windows.Forms.View.Details;
			this.listViewRCC.AfterLabelEdit += new System.Windows.Forms.LabelEditEventHandler(this.listViewRCC_AfterLabelEdit);
			this.listViewRCC.BeforeLabelEdit += new System.Windows.Forms.LabelEditEventHandler(this.listViewRCC_BeforeLabelEdit);
			this.listViewRCC.SelectedIndexChanged += new System.EventHandler(this.listViewRCC_SelectedIndexChanged);
			// 
			// columnHeader3
			// 
			this.columnHeader3.Text = "Extended Result Code";
			this.columnHeader3.Width = 185;
			// 
			// columnHeader4
			// 
			this.columnHeader4.Text = "Color";
			this.columnHeader4.Width = 111;
			// 
			// panel3
			// 
			this.panel3.Controls.Add(this.label4);
			this.panel3.Controls.Add(this.buttonEditRCC);
			this.panel3.Controls.Add(this.buttonRemoveRCC);
			this.panel3.Controls.Add(this.buttonAddRCC);
			this.panel3.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panel3.Location = new System.Drawing.Point(8, 262);
			this.panel3.Name = "panel3";
			this.panel3.Size = new System.Drawing.Size(314, 64);
			this.panel3.TabIndex = 1;
			// 
			// buttonEditRCC
			// 
			this.buttonEditRCC.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.buttonEditRCC.Location = new System.Drawing.Point(184, 32);
			this.buttonEditRCC.Name = "buttonEditRCC";
			this.buttonEditRCC.TabIndex = 14;
			this.buttonEditRCC.Text = "Edit Color...";
			this.buttonEditRCC.Click += new System.EventHandler(this.buttonEditRCC_Click);
			// 
			// buttonRemoveRCC
			// 
			this.buttonRemoveRCC.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.buttonRemoveRCC.Location = new System.Drawing.Point(96, 32);
			this.buttonRemoveRCC.Name = "buttonRemoveRCC";
			this.buttonRemoveRCC.TabIndex = 13;
			this.buttonRemoveRCC.Text = "Remove";
			this.buttonRemoveRCC.Click += new System.EventHandler(this.buttonRemoveRCC_Click);
			// 
			// buttonAddRCC
			// 
			this.buttonAddRCC.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.buttonAddRCC.Location = new System.Drawing.Point(8, 32);
			this.buttonAddRCC.Name = "buttonAddRCC";
			this.buttonAddRCC.TabIndex = 12;
			this.buttonAddRCC.Text = "Add";
			this.buttonAddRCC.Click += new System.EventHandler(this.buttonAddRCC_Click);
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(8, 8);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(280, 16);
			this.label3.TabIndex = 12;
			this.label3.Text = "Edit experiment names by clicking on the label.";
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(8, 8);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(280, 16);
			this.label4.TabIndex = 15;
			this.label4.Text = "Edit result code by clicking on the label.";
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(16, 304);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(296, 16);
			this.label5.TabIndex = 17;
			this.label5.Text = "The first matching pattern, starting from top, will be used.";
			// 
			// AdvancedOptionsForm
			// 
			this.AcceptButton = this.okbutton;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.cancelbutton;
			this.ClientSize = new System.Drawing.Size(346, 410);
			this.Controls.Add(this.tabControl1);
			this.Controls.Add(this.okbutton);
			this.Controls.Add(this.cancelbutton);
			this.DockPadding.All = 4;
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "AdvancedOptionsForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "DCL Add-In Advanced Options";
			this.tabControl1.ResumeLayout(false);
			this.tabPage1.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.numericUpDown1)).EndInit();
			this.tabPage2.ResumeLayout(false);
			this.tabPage3.ResumeLayout(false);
			this.panel2.ResumeLayout(false);
			this.tabPage4.ResumeLayout(false);
			this.panel3.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void cancelbutton_Click(object sender, System.EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
		}

		private void okbutton_Click(object sender, System.EventArgs e)
		{
			this.config.RefreshInterval = (int)this.numericUpDown1.Value;

			this.config.ErrorPatterns.Clear();

			foreach(CompilerErrorPattern cep in this.errorPatterns)
			{
				CompilerErrorPattern copy = new CompilerErrorPattern(cep);
				this.config.ErrorPatterns.Add(cep);
			}

			this.config.CodeLinesOverhead.Clear();

			foreach(string experiment in this.codeLinesOverhead.Keys)
			{
				this.config.CodeLinesOverhead.Add(experiment, this.codeLinesOverhead[experiment]);
			}

			this.config.ResultCodeColors.Clear();

			foreach(string resultCode in this.resultCodeColors.Keys)
			{
				this.config.ResultCodeColors.Add(resultCode, this.resultCodeColors[resultCode]);
			}

			this.DialogResult = DialogResult.OK;
		}

		private void buttonMoveUp_Click(object sender, System.EventArgs e)
		{
			if (this.listBoxPatterns.SelectedIndex >= 1)
			{
				this.listBoxPatterns.BeginUpdate();
				this.errorPatterns.Reverse(this.listBoxPatterns.SelectedIndex - 1, 2);
				this.updateListbox();
				this.listBoxPatterns.SelectedIndex--;
				this.listBoxPatterns.EndUpdate();
			}
		}

		private void buttonMoveDown_Click(object sender, System.EventArgs e)
		{
			if (this.listBoxPatterns.SelectedIndex < this.errorPatterns.Count-1)
			{
				this.listBoxPatterns.BeginUpdate();
				this.errorPatterns.Reverse(this.listBoxPatterns.SelectedIndex, 2);
				this.updateListbox();
				this.listBoxPatterns.SelectedIndex++;
				this.listBoxPatterns.EndUpdate();
			}
		}

		private void updateListboxBegin()
		{
			this.listBoxPatterns.DataSource = null;
		}

		private void updateListboxEnd()
		{
			this.listBoxPatterns.DataSource = this.errorPatterns;
			this.listBoxPatterns.DisplayMember = "Name";
		}

		private void updateListbox()
		{
			this.updateListboxBegin();
			this.updateListboxEnd();
		}

		private void buttonEditPattern_Click(object sender, System.EventArgs e)
		{
			if (this.listBoxPatterns.SelectedIndex >= 0)
			{
				ErrorPatternForm epf = new ErrorPatternForm(
					(CompilerErrorPattern) this.errorPatterns[this.listBoxPatterns.SelectedIndex]);

				if (epf.ShowDialog() == DialogResult.OK)
				{
					this.listBoxPatterns.BeginUpdate();
					this.errorPatterns[this.listBoxPatterns.SelectedIndex] = epf.ErrorPattern;
					this.updateListbox();
					this.listBoxPatterns.EndUpdate();
				}
			}
		}

		private void buttonRemovePattern_Click(object sender, System.EventArgs e)
		{
			// a hack: we don't let the user remove all items to avoid the problems with ListBox.DataSource
			if (this.listBoxPatterns.SelectedIndex >= 0 && this.errorPatterns.Count > 1)
			{
				if (MessageBox.Show("Do you really want to remove the error pattern '" + this.listBoxPatterns.Text + "'?",
					"Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
				{
					this.listBoxPatterns.BeginUpdate();
					int sel = this.listBoxPatterns.SelectedIndex;

					this.listBoxPatterns.SelectedIndex = -1;
					this.updateListboxBegin();
					this.errorPatterns.RemoveAt(sel);
					this.updateListboxEnd();
					if (this.errorPatterns.Count > 0)
						this.listBoxPatterns.SelectedIndex = sel - 1;
					else
						this.listBoxPatterns.SelectedIndex = -1;
					this.listBoxPatterns.EndUpdate();
				}
			}
		}

		private void buttonAddPattern_Click(object sender, System.EventArgs e)
		{
			ErrorPatternForm epf = new ErrorPatternForm(null);

			if (epf.ShowDialog() == DialogResult.OK)
			{
				this.listBoxPatterns.BeginUpdate();
				this.errorPatterns.Add(epf.ErrorPattern);
				this.updateListbox();
				this.listBoxPatterns.EndUpdate();
			}
		}

		private void listBoxPatterns_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			bool enabled = (this.listBoxPatterns.SelectedIndex >= 0);

			this.buttonEditPattern.Enabled = enabled;
			this.buttonRemovePattern.Enabled = enabled;
			this.buttonMoveUp.Enabled = enabled;
			this.buttonMoveDown.Enabled = enabled;
		}

		private void buttonAddCLO_Click(object sender, System.EventArgs e)
		{
			string experiment = InputBoxForm.ShowInputBox("Please enter experiment name", "Experiment:", "");

			if (experiment != null)
			{
				if (this.codeLinesOverhead.Contains(experiment))
				{
					MessageBox.Show("A setting for this experiment already exists.", "Error");
				}
				else
				{
					string input = InputBoxForm.ShowInputBox("Please enter line number", "No. of lines added before the user code:", "0");

					if (input != null)
					{
						int number = 0;

						try
						{
							number = System.Convert.ToInt32(input);
						}
						catch(SystemException)
						{
							MessageBox.Show("Please enter a number!", "Warning");
							number = 0;
							input = "0";
						}

						this.codeLinesOverhead.Add(experiment,number);

						ListViewItem item = new ListViewItem(experiment);
						item.SubItems.Add(input);
						this.listViewCLO.Items.Add(item);
					}
				}
			}
		}

		private void buttonRemoveCLO_Click(object sender, System.EventArgs e)
		{
			if (MessageBox.Show("Do you really want to remove the selected setting for '" + this.listViewCLO.SelectedItems[0].SubItems[0].Text + "'?",
				"Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
			{
				string experiment = this.listViewCLO.SelectedItems[0].SubItems[0].Text;
				this.listViewCLO.Items.RemoveAt(this.listViewCLO.SelectedIndices[0]);

				this.codeLinesOverhead.Remove(experiment);
			}
		}

		private void buttonEditCLO_Click(object sender, System.EventArgs e)
		{
			if (this.listViewCLO.SelectedItems.Count > 0)
			{
				string experiment = this.listViewCLO.SelectedItems[0].Text;
				string input = InputBoxForm.ShowInputBox("Please enter line number",
					"No. of lines added before the user code:", System.Convert.ToString((int) this.codeLinesOverhead[experiment]));

				if (input != null)
				{
					try
					{
						int number = System.Convert.ToInt32(input);
						this.codeLinesOverhead[experiment] = number;
						this.listViewCLO.SelectedItems[0].SubItems[1].Text = input;
					}
					catch(SystemException)
					{
						MessageBox.Show("Please enter a number!", "Warning");
					}
				}
			}
		}

		private string oldValue = null;

		private void listViewCLO_BeforeLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
		{
			this.oldValue = this.listViewCLO.Items[e.Item].SubItems[0].Text;
		}

		private void listViewCLO_AfterLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
		{
			if (!e.CancelEdit && e.Label != null)
			{
				string newValue = e.Label;
				if (this.oldValue != null && this.oldValue != newValue)
				{
					int number = (int) this.codeLinesOverhead[this.oldValue];
					this.codeLinesOverhead.Remove(this.oldValue);

					this.codeLinesOverhead.Add(newValue, number);
				}
			}
		}

		private void listViewCLO_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			bool enabled = (this.listViewCLO.SelectedIndices.Count > 0);

			this.buttonEditCLO.Enabled = enabled;
			this.buttonRemoveCLO.Enabled = enabled;
		}

		private void buttonAddRCC_Click(object sender, System.EventArgs e)
		{
			string resultCode = InputBoxForm.ShowInputBox("Please enter result code", "Result Code:", "");

			if (resultCode != null)
			{
				if (this.resultCodeColors.Contains(resultCode))
				{
					MessageBox.Show("A setting for this result code already exists.", "Error");
				}
				else
				{
					if (this.colorDialog1.ShowDialog() == DialogResult.OK)
					{
						this.resultCodeColors.Add(resultCode, this.colorDialog1.Color);

						ListViewItem item = new ListViewItem(resultCode);
						item.ForeColor = this.colorDialog1.Color;
						item.SubItems.Add(this.colorDialog1.Color.Name);
						this.listViewRCC.Items.Add(item);
					}
				}
			}
		}

		private void buttonRemoveRCC_Click(object sender, System.EventArgs e)
		{
			if (MessageBox.Show("Do you really want to remove the selected setting for '" + this.listViewRCC.SelectedItems[0].SubItems[0].Text + "'?",
				"Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
			{
				string resultCode = this.listViewRCC.SelectedItems[0].SubItems[0].Text;
				this.listViewRCC.Items.RemoveAt(this.listViewRCC.SelectedIndices[0]);

				this.resultCodeColors.Remove(resultCode);
			}
		}

		private void buttonEditRCC_Click(object sender, System.EventArgs e)
		{
			if (this.listViewRCC.SelectedItems.Count > 0)
			{
				string resultCode = this.listViewRCC.SelectedItems[0].Text;
				this.colorDialog1.Color = (Color) this.resultCodeColors[resultCode];

				if (this.colorDialog1.ShowDialog() == DialogResult.OK)
				{
					this.resultCodeColors[resultCode] = this.colorDialog1.Color;
					this.listViewRCC.SelectedItems[0].ForeColor = this.colorDialog1.Color;
					this.listViewRCC.SelectedItems[0].SubItems[1].Text = this.colorDialog1.Color.Name;
				}
			}
		}

		private void listViewRCC_BeforeLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
		{
			this.oldValue = this.listViewRCC.Items[e.Item].SubItems[0].Text;
		}

		private void listViewRCC_AfterLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
		{
			if (!e.CancelEdit && e.Label != null)
			{
				string newValue = e.Label;
				if (this.oldValue != null && this.oldValue != newValue)
				{
					Color color = (Color) this.resultCodeColors[this.oldValue];
					this.resultCodeColors.Remove(this.oldValue);

					this.resultCodeColors.Add(newValue, color);
				}
			}
		}

		private void listViewRCC_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			bool enabled = (this.listViewRCC.SelectedIndices.Count > 0);

			this.buttonEditRCC.Enabled = enabled;
			this.buttonRemoveRCC.Enabled = enabled;
		}
	}
}