using System;
using System.Collections;
using System.Text.RegularExpressions;

namespace DCLJobView
{
	/// <summary>
	/// Can parse the compiler output and produce a list of errors and warnings.
	/// Idea and basic algorithm stolen from jEdit Console plugin.
	/// 
	/// Use CompilerErrorPattern to create a pattern set.
	/// </summary>
	public class CompilerOutputParser
	{
		private ArrayList patterns;
		private Hashtable codeLinesOverhead;

		public CompilerOutputParser(ArrayList patterns, Hashtable codeLinesOverhead)
		{
			// Note: the order is important! Patterns at the top have a higher priority
			this.patterns = patterns;
			this.codeLinesOverhead = codeLinesOverhead;
		}

		public CompilerOutputItem[] Parse(string experiment, string ticket, string output)
		{
			if (output == null || output == "")
				return new CompilerOutputItem[0];

			ArrayList items = new ArrayList();

			int overhead;
			try
			{
				overhead = (int) this.codeLinesOverhead[experiment];
			}
			catch (SystemException)
			{
				overhead = 0;
			}

			// split the output string into separate lines
			string[] lines;

			if (output.IndexOf("\r\n") >= 0)
			{
				Regex newline = new Regex("\r\n");
				lines = newline.Split(output);
			}
			else
			{
				lines = output.Split('\n');
			}

			CompilerErrorPattern lastMatcher = null;
			CompilerOutputItem lastItem = null;

			// we look at the output line per line
			foreach (string line in lines)
			{
				if (lastItem != null)
				{
					string extraMessage = null;

					if (lastMatcher != null && lastMatcher.Match(line) == null)
						extraMessage = lastMatcher.MatchExtra(line);

					if (extraMessage != null)
					{
						lastItem.ExtraMessage = extraMessage;
						lastItem = null;
						continue;
					}
					else
					{
						lastItem.Line -= overhead;
						lastItem.Ticket = ticket;
						items.Add(lastItem);
						lastMatcher = null;
						lastItem = null;
					}
				}

				foreach (CompilerErrorPattern p in this.patterns)
				{
					CompilerOutputItem item = p.Match(line);

					if (item != null)
					{
						lastMatcher = p;
						lastItem = item;
					}
				}
			}

			// this is necessary if we encounter an error in the last line of the output
			if (lastItem != null)
			{
				lastItem.Line -= overhead;
				items.Add(lastItem);
			}

			return (CompilerOutputItem[]) items.ToArray(typeof(CompilerOutputItem));
		}
	}
}
