using System;
using System.ComponentModel;
using System.Collections;
using System.Drawing;

namespace DCLJobView
{
	[Serializable]
	public class Configuration
	{
		private string login;
		private string password;
		private string _DCLExperimentService;
		private string _DCLTicketService;
		private string configFile;
		private string webFrontEndURL;
		private int refresh;
		private ArrayList errorPatterns;     // objects inside should be CompilerErrorPattern
		private Hashtable codeLinesOverhead; // string -> int
		private Hashtable resultCodeColors;  // string -> System.Drawing.Color

		public Configuration(string configFile) 
		{
			this.login = "";
			this.password = "";
			this.DCLExperimentService = "http://was.discourse.de/dclv2/dclexperimentservice/dclexperimentservice.asmx";
			this.DCLTicketService = "http://was.discourse.de/dclv2/dclticketserver/dclticketserver.asmx";
			this.webFrontEndURL = "http://was.discourse.de/discourse";
			this.configFile = configFile;
			this.refresh = 5;

			this.errorPatterns = new ArrayList();
			this.errorPatterns.Add(
				new CompilerErrorPattern(
				"MS .NET",
				@"^(?<filename>.+)\((?<line>\d+),(?<column>\d+)\):\s+(?<message>.+)$",
				@"",
				@"") );

			this.errorPatterns.Add(
				new CompilerErrorPattern(
				"MS Visual C++",
				@"(?<filename>[^\(]*)[\(](?<line>\d+)[)].:.([^:]*):\s*(?<message>.*)",
				@"(?<filename>[^\(]*)[\(](?<line>\d+)[)].:.(warning[^:]*):\s*(?<message>.+)",
				@"\s+(?<extra>.*)") );

			this.errorPatterns.Add(
				new CompilerErrorPattern(
				"Generic",
				@"(?<filename>.+?):(?<line>\d+):(?<message>.+)",
				@"(?<filename>[^\(]+):(?<line>\d+):\s*warning:\s*(?<message>.+)",
				@"\s*(?<extra>(?:symbol|location|found|required)\s*:.*)") );

			this.codeLinesOverhead = new Hashtable();
			this.codeLinesOverhead.Add("Hau den Lukas - Perform API",0);
			this.codeLinesOverhead.Add("Pendulum",98);
			this.codeLinesOverhead.Add("Lego Simulator",1);
			this.codeLinesOverhead.Add("Lego_Robots",31);

			this.resultCodeColors = new Hashtable();
			this.resultCodeColors.Add("Failed",Color.Red);
			this.resultCodeColors.Add("Experiment crashed",Color.Red);
			this.resultCodeColors.Add("Stop Call Failed",Color.Red);
			this.resultCodeColors.Add("StopSignal",Color.Green);
			this.resultCodeColors.Add("ready",Color.Green);
			this.resultCodeColors.Add("Successful",Color.Green);
			this.resultCodeColors.Add("",Color.Green);
		}

		[CategoryAttribute("User Settings"),
		 DescriptionAttribute("The login name for the DCL user.")]
		public string Login
		{
			get { return this.login; }
			set { this.login = value; }
		}

		[CategoryAttribute("User Settings"),
		 DescriptionAttribute("The password assigned with the login name of the user.")]
		public string Password
		{
			get { return this.password; }
			set { this.password = value; }
		}

		[CategoryAttribute("Service Settings"),
		 DescriptionAttribute("The URL of the DCL Experiment Web Service.")]
		public string DCLExperimentService
		{
			get { return this._DCLExperimentService; }
			set { this._DCLExperimentService = value; }
		}

		[CategoryAttribute("Service Settings"),
		 DescriptionAttribute("The URL of the DCL Ticket Web Service.")]
		public string DCLTicketService
		{
			get { return this._DCLTicketService; }
			set { this._DCLTicketService = value; }
		}

		[CategoryAttribute("Service Settings"),
		DescriptionAttribute("The URL of the DCL Web Front End.")]
		public string WebFrontEndURL
		{
			get { return this.webFrontEndURL; }
			set { this.webFrontEndURL = value; }
		}

		[Browsable(false),
		 CategoryAttribute("Internal Settings"),
		 DescriptionAttribute("The path and name of the users config file.")]
		public string ConfigFile
		{
			get { return this.configFile; }
			set { this.configFile = value; }
		}

		[CategoryAttribute("Add-In Settings"),
		DescriptionAttribute("The refresh intervall for the DCL Add-In Job View.")]
		public int RefreshInterval
		{
			get { return this.refresh; }
			set { this.refresh = value; }
		}

		[CategoryAttribute("User Settings"),
		DescriptionAttribute("A list of error patterns for parsing of compiler output.")]
		public ArrayList ErrorPatterns
		{
			get { return this.errorPatterns; }
			set { this.errorPatterns = value; }
		}

		[CategoryAttribute("User Settings"),
		DescriptionAttribute("Maps experiment names to the number (int) of lines which are added before the sent code.")]
		public Hashtable CodeLinesOverhead
		{
			get { return this.codeLinesOverhead; }
			set { this.codeLinesOverhead = value; }
		}

		[CategoryAttribute("User Settings"),
		DescriptionAttribute("Maps extended result codes to Colors in which the text of the job node is displayed.")]
		public Hashtable ResultCodeColors
		{
			get { return this.resultCodeColors; }
			set { this.resultCodeColors = value; }
		}
	}
}