using System;
using System.IO;
using System.Data;
using System.Drawing;
using System.Threading;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Web.Services.Protocols;
using System.Runtime.InteropServices;

using DCLJobView.TreeModel;
using DCLJobView.Events;

namespace DCLJobView
{
	public class JobViewControl : System.Windows.Forms.UserControl
	{
		#region Static fields
		public static int RUN_JOB_IMAGE             = 0;
		public static int CANCEL_JOB_IMAGE          = 1;
		public static int SHOW_OPTIONS_IMAGE        = 2;
		public static int SHOW_RESULTS_IMAGE        = 3;
		public static int EXPERIMENT_TYPE_IMAGE     = 4;
		public static int JOB_RUNNING_IMAGE         = 5;
		public static int JOB_FINISHED_IMAGE        = 6;
		public static int CODE_RESULT_IMAGE         = 7;
		public static int TEXT_RESULT_IMAGE         = 8;
		public static int BINARY_RESULT_IMAGE       = 9;
		public static int COMPILER_RESULT_IMAGE     = 10;
		public static int NON_EXISTING_EXPERIMENT_TYPE_IMAGE = 11;

		public static Color LastSentHighlightColor = Color.FromArgb(255,255,64); //SystemColors.GrayText SystemColors.Info
		#endregion

		#region Private fields
		private System.Windows.Forms.ToolBar toolBar1;
		private System.Windows.Forms.ImageList imageList1;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.TreeView treeView1;
		private System.Windows.Forms.ToolBarButton runAsJobButton;
		private System.Windows.Forms.ToolBarButton cancelJobButton;
		private System.Windows.Forms.ToolBarButton showResultsWebButton;
		private System.Windows.Forms.ToolBarButton separatorButton;
		private System.Windows.Forms.ToolBarButton optionsButton;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ContextMenu contextMenu1;

		private Configuration config;
		private Thread updateThread;
		private CompilerOutputParser compilerOutputParser;

		private DCLJobView.DCLExperimentService.DCLExperimentService experimentWebService;
		private DCLJobView.DCLTicketServer.DCLTicketServer ticketWebService;
		#endregion
		
		#region Constructor and other important methods
		public JobViewControl()
		{
			InitializeComponent();

			this.compilerOutputParser = null;

			this.LastSentJobTicket = "";
			this.LastSentJobNode = null;
			this.LastSentExperimentNode = null;

			// create web service stubs
			this.experimentWebService = new DCLJobView.DCLExperimentService.DCLExperimentService();
			this.ticketWebService = new DCLJobView.DCLTicketServer.DCLTicketServer();
		}

		protected override void OnLoad(EventArgs e)
		{
			this.loadConfiguration();

			base.OnLoad(e);
		}


		protected override void Dispose( bool disposing )
		{
			try
			{
				if (this.updateThread != null && this.updateThread.IsAlive)
				{
					this.updateThread.Interrupt();
					this.updateThread.Abort(AbortType.EXIT);

					// join ensures that we wait for the thread to finish
					//this.updateThread.Join();
				}
			}
			catch(ThreadStateException)
			{
				// for some reason we get a ThreadStateException although Thread.Abort() shouldn't throw any
				this.updateThread.Abort(AbortType.EXIT);
			}

			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		private void setServiceURLs()
		{
			try
			{
				this.experimentWebService.Url = this.config.DCLExperimentService;
				this.ticketWebService.Url = this.config.DCLTicketService;
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
				this.DoLogMessage("Configuration error: " + e.Message, false);
			}
		}
		#endregion

		#region Vom Komponenten-Designer generierter Code
		/// <summary> 
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(JobViewControl));
			this.toolBar1 = new System.Windows.Forms.ToolBar();
			this.runAsJobButton = new System.Windows.Forms.ToolBarButton();
			this.cancelJobButton = new System.Windows.Forms.ToolBarButton();
			this.showResultsWebButton = new System.Windows.Forms.ToolBarButton();
			this.separatorButton = new System.Windows.Forms.ToolBarButton();
			this.optionsButton = new System.Windows.Forms.ToolBarButton();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			this.panel1 = new System.Windows.Forms.Panel();
			this.treeView1 = new System.Windows.Forms.TreeView();
			this.contextMenu1 = new System.Windows.Forms.ContextMenu();
			this.panel1.SuspendLayout();
			this.SuspendLayout();
			// 
			// toolBar1
			// 
			this.toolBar1.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.toolBar1.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																						this.runAsJobButton,
																						this.cancelJobButton,
																						this.showResultsWebButton,
																						this.separatorButton,
																						this.optionsButton});
			this.toolBar1.DropDownArrows = true;
			this.toolBar1.ImageList = this.imageList1;
			this.toolBar1.Location = new System.Drawing.Point(0, 0);
			this.toolBar1.Name = "toolBar1";
			this.toolBar1.ShowToolTips = true;
			this.toolBar1.Size = new System.Drawing.Size(272, 28);
			this.toolBar1.TabIndex = 1;
			this.toolBar1.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.toolBar1_ButtonClick);
			// 
			// runAsJobButton
			// 
			this.runAsJobButton.ImageIndex = 0;
			this.runAsJobButton.ToolTipText = "Sends a job to the DCL to execute it.";
			// 
			// cancelJobButton
			// 
			this.cancelJobButton.Enabled = false;
			this.cancelJobButton.ImageIndex = 1;
			this.cancelJobButton.ToolTipText = "Cancels the selected job.";
			// 
			// showResultsWebButton
			// 
			this.showResultsWebButton.Enabled = false;
			this.showResultsWebButton.ImageIndex = 3;
			this.showResultsWebButton.ToolTipText = "Show DCL Job results in webpage.";
			// 
			// separatorButton
			// 
			this.separatorButton.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// optionsButton
			// 
			this.optionsButton.ImageIndex = 2;
			this.optionsButton.ToolTipText = "Shows the options dialog for the DCL extension add-in.";
			// 
			// imageList1
			// 
			this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.treeView1);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panel1.Location = new System.Drawing.Point(0, 28);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(272, 404);
			this.panel1.TabIndex = 2;
			// 
			// treeView1
			// 
			this.treeView1.ContextMenu = this.contextMenu1;
			this.treeView1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.treeView1.HideSelection = false;
			this.treeView1.ImageList = this.imageList1;
			this.treeView1.Location = new System.Drawing.Point(0, 0);
			this.treeView1.Name = "treeView1";
			this.treeView1.Size = new System.Drawing.Size(272, 404);
			this.treeView1.TabIndex = 1;
			this.treeView1.MouseDown += new System.Windows.Forms.MouseEventHandler(this.treeView1_MouseDown);
			this.treeView1.DoubleClick += new System.EventHandler(this.treeView1_DoubleClick);
			this.treeView1.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.treeView1_AfterSelect);
			// 
			// contextMenu1
			// 
			this.contextMenu1.Popup += new System.EventHandler(this.contextMenu1_Popup);
			// 
			// JobViewControl
			// 
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.toolBar1);
			this.Name = "JobViewControl";
			this.Size = new System.Drawing.Size(272, 432);
			this.panel1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Event-listening methods (from child controls)
		private void toolBar1_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if(e.Button.Equals(this.runAsJobButton))
			{
				this.OnSendJobClicked(new EventArgs());
			}

			if(e.Button.Equals(this.cancelJobButton))
			{
				if(this.treeView1.SelectedNode is Job)
				{
					if (((Job) this.treeView1.SelectedNode).Finished)
						this.deleteJob();
					else
						this.cancelJob();
				}
			}

			if(e.Button.Equals(this.showResultsWebButton))
			{
				this.ShowWebResult();
			}

			if(e.Button.Equals(this.optionsButton))
			{
				this.showOptions();
			}				
		}

		private void contextMenu1_Popup(object sender, System.EventArgs e)
		{
			// clear popup menu
			this.contextMenu1.MenuItems.Clear();

			if(this.treeView1.SelectedNode != null)
			{
				if(this.treeView1.SelectedNode is ExperimentType)
				{
					// add node specific buttons
					MenuItem startAsExperimentItem = new MenuItem("Send Job");
					startAsExperimentItem.Click += new EventHandler(this.startAsExperimentItem_Clicked);
					this.contextMenu1.MenuItems.Add(startAsExperimentItem);
				}

				if(this.treeView1.SelectedNode is Job)
				{
					// add node specific buttons
					MenuItem showWebResultItem = new MenuItem("Show Results in Webpage");
					showWebResultItem.Click += new EventHandler(this.showWebResultItem_Clicked);
					this.contextMenu1.MenuItems.Add(showWebResultItem);

					if(((Job)this.treeView1.SelectedNode).Finished)
					{
						MenuItem deleteJobItem = new MenuItem("Delete Ticket");
						deleteJobItem.Click += new EventHandler(this.deleteJobItem_Clicked);
						this.contextMenu1.MenuItems.Add(deleteJobItem);
					}
					else
					{
						MenuItem cancelJobItem = new MenuItem("Cancel Job");
						cancelJobItem.Click += new EventHandler(this.cancelJobItem_Clicked);
						this.contextMenu1.MenuItems.Add(cancelJobItem);
					}

					MenuItem showTicketItem = new MenuItem("Show Ticket ID");
					showTicketItem.Click += new EventHandler(this.showTicketItem_Clicked);
					this.contextMenu1.MenuItems.Add(showTicketItem);

				}

				if(this.treeView1.SelectedNode is Result)
				{
					// add node specific buttons
					Result result = (Result)this.treeView1.SelectedNode;

					if(!result.IsBinary)
					{
						if(this.ShowStringResultClicked != null)
						{
							MenuItem showResultItem = new MenuItem("Show Result");
							showResultItem.Click += new EventHandler(this.showResultItem_Clicked);
							this.contextMenu1.MenuItems.Add(showResultItem);
						}
					}
					else
					{
						if(this.ShowBinaryResultClicked != null)
						{
							MenuItem showResultItem = new MenuItem("Show Result");
							showResultItem.Click += new EventHandler(this.showResultItem_Clicked);
							this.contextMenu1.MenuItems.Add(showResultItem);
						}
					}

					/*NOTE: this item is not useful here because 
							it shows always all results of a job*/

//					MenuItem showWebResultItem = new MenuItem("Show Results in Webpage");
//					showWebResultItem.Click += new EventHandler(this.showWebResultItem_Clicked);
//					this.contextMenu1.MenuItems.Add(showWebResultItem);
				}
			}

			if(this.contextMenu1.MenuItems.Count > 0)
			{
				// add some additional buttons
				MenuItem separatorItem = new MenuItem("-");
				this.contextMenu1.MenuItems.Add(separatorItem);
			}

			MenuItem optionItem = new MenuItem("Options...");
			optionItem.Click += new EventHandler(this.optionItem_Clicked);
			this.contextMenu1.MenuItems.Add(optionItem);
		}

		private void treeView1_DoubleClick(object sender, System.EventArgs e)
		{
			if(this.treeView1.SelectedNode != null)
			{
				if(this.treeView1.SelectedNode is Result)
				{
					if (((Result)this.treeView1.SelectedNode).IsBinary)
					{
						if(this.ShowBinaryResultClicked != null)
							this.showResult();
						else if(this.ShowWebResultClicked != null)
							this.ShowWebResult();
					}
					else
					{
						if(this.ShowStringResultClicked != null)
							this.showResult();
					}
				}
			}
		}

		private void startAsExperimentItem_Clicked(object sender, EventArgs e)
		{
			this.OnSendJobClicked(new EventArgs());
		}

		private void cancelJobItem_Clicked(object sender, EventArgs e)
		{
			this.cancelJob();
		}

		private void deleteJobItem_Clicked(object sender, EventArgs e)
		{
			this.deleteJob();
		}

		private void showTicketItem_Clicked(object sender, EventArgs e)
		{
			this.showTicket();
		}

		private void showResultItem_Clicked(object sender, EventArgs e)
		{
			this.showResult();
		}

		private void showWebResultItem_Clicked(object sender, EventArgs e)
		{
			this.ShowWebResult();
		}

		private void optionItem_Clicked(object sender, EventArgs e)
		{
			this.showOptions();
		}

		private void treeView1_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(this.treeView1.GetNodeAt(e.X, e.Y) == null)
			{
				this.treeView1.SelectedNode = null;
				this.treeView1_AfterSelect(sender, new TreeViewEventArgs(null));
			}
			else if (e.Button == MouseButtons.Right)
				this.treeView1.SelectedNode = this.treeView1.GetNodeAt(e.X, e.Y);
		}

		private void treeView1_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			if (this.treeView1.SelectedNode != null)
			{
				if (this.treeView1.SelectedNode is ExperimentType)
				{
					this.cancelJobButton.Enabled = false;
					this.showResultsWebButton.Enabled = false;
				}
				else if (this.treeView1.SelectedNode is Job)
				{
					this.cancelJobButton.Enabled = true;
					this.showResultsWebButton.Enabled = true;

					if (((Job) this.treeView1.SelectedNode).Finished)
						this.cancelJobButton.ToolTipText = "Deletes the selected ticket.";
					else
						this.cancelJobButton.ToolTipText = "Cancels the selected job.";
				}
				else if (this.treeView1.SelectedNode is Result)
				{
					this.cancelJobButton.Enabled = false;
					this.showResultsWebButton.Enabled = true;
				}
			}
			else
			{
				this.cancelJobButton.Enabled = false;
				this.showResultsWebButton.Enabled = false;
			}
		}
		#endregion

		#region Private methods
		/// <summary>
		///		Shows the result of a job.
		/// </summary>
		private void showResult()
		{
			if (this.treeView1.SelectedNode is Result)
			{
				Result result = (Result)this.treeView1.SelectedNode;

				string type = result.ResultType;
				string name = result.Name;
				string ticket = result.Job.Ticket;
			
				if (result.IsBinary)
				{
					try
					{
						byte[] resultData;
						resultData = this.experimentWebService.GetResult(ticket, name);
						this.OnShowBinaryResultClicked(new BinaryResultEvent(name, type, ticket, resultData));
					}
					catch(SoapException e)
					{
						Trace.WriteLine(e.Message);
						this.DoLogMessage("Web Exception: " + e.Message, false);
						return;
					}
				}
				else
				{
					try
					{
						string resultString = "";
						resultString = this.experimentWebService.GetStringResult(ticket, name);

						if (result.Name == "Compiler Output")
						{
							CompilerOutputItem[] items = compilerOutputParser.Parse(getExperimentType(ticket).Name, ticket, resultString);
							this.OnShowCompilerResultClicked(new CompilerResultEvent(name, type, ticket, items, resultString));
						}
						else
						{
							this.OnShowStringResultClicked(new StringResultEvent(name, type, ticket, resultString));
						}
					}
					catch(SoapException e)
					{
						Trace.WriteLine(e.Message);
						this.DoLogMessage("Web Exception: " + e.Message, false);
						return;
					}
				}
			}
		}

		/// <summary>
		///		Show web result.
		/// </summary>
		private void ShowWebResult()
		{
			if(this.treeView1.SelectedNode != null)
			{
				if(this.treeView1.SelectedNode is Job || this.treeView1.SelectedNode is Result)
				{
					Job job;

					if(this.treeView1.SelectedNode is Job)
					{
						job = (Job)this.treeView1.SelectedNode;
					}
					else //if(this.treeView1.SelectedNode is Result)
					{
						job = ((Result)this.treeView1.SelectedNode).Job;
					}
					
					// NOTE: this web frontend url is probably very DCL specific
					string url = this.config.WebFrontEndURL + "/jobdetails.aspx?ticket=" + job.Ticket;
					this.OnShowWebResultClicked(new WebResultEvent("all", "web", job.Ticket, url));
				}
			}
		}
		#endregion

		#region Protected Event raise methods (OnEvent...)
		protected virtual void OnLogMessage(MessageEvent e)
		{
			if (LogMessage != null)
			{
				LogMessage(this, e);
			}
		}

		protected virtual void OnSendJobClicked(EventArgs e)
		{
			if (SendJobClicked != null)
			{
				SendJobClicked(this, e);
			}
		}

		protected virtual void OnShowStringResultClicked(StringResultEvent e)
		{
			if (ShowStringResultClicked != null)
			{
				ShowStringResultClicked(this, e);
			}
		}

		protected virtual void OnShowCompilerResultClicked(CompilerResultEvent e)
		{
			if (ShowCompilerResultClicked != null)
			{
				ShowCompilerResultClicked(this, e);
			}
		}

		protected virtual void OnShowBinaryResultClicked(BinaryResultEvent e)
		{
			if (ShowBinaryResultClicked != null)
			{
				ShowBinaryResultClicked(this, e);
			}
		}

		protected virtual void OnShowWebResultClicked(WebResultEvent e)
		{
			if (ShowWebResultClicked != null)
			{
				ShowWebResultClicked(this, e);
			}
		}
		#endregion
		
		#region Update-Thread related Private methods
		private enum AbortType {EXIT, STOP, CONFIG_CHANGED};

		private void updateExperiments()
		{
			// get experiment list
			string[] experiments = null;
			try
			{
				experiments = this.experimentWebService.ListExperiments();
			}
			catch(SoapException e)
			{
				Trace.WriteLine(e.Message);
				this.DoLogMessage("Web Exception: " + e.Message, true);
				return;
			}

			// check error code
			if(experiments == null)
			{
				this.DoLogMessage("No experiment registered.", true);
				return;
			}

			/* if an experiment deletes himself in HandleExperimentList() the collection
				   of the foreach loop gets corrupted; thus we cache it */
			ArrayList experimentList = new ArrayList(experiments);
			TreeNode[] nodesCache = new TreeNode[this.treeView1.Nodes.Count];
			this.treeView1.Nodes.CopyTo(nodesCache,0);

			foreach(TreeNode node in nodesCache)
			{
				if(node is ExperimentType)
				{
					((ExperimentType)node).HandleExperimentList(experimentList, this);
				}
			}

			try
			{
				foreach(string experiment in experimentList)
				{
					ExperimentType expType = new ExperimentType(experiment, true, this);
					this.treeView1.Invoke(new AddNodeHandler(this.addNodeCallback),
						new object[] { expType, this.treeView1.Nodes });
				}
			}
			// this happens in Invoke when no handle is created for the treeview control
			catch(InvalidOperationException)
			{}
		}

		private void updateJobs()
		{
			string[] tickets = null;
			try
			{
				bool result;
				tickets = this.ticketWebService.GetTicketList(this.config.Login, this.config.Password, out result);
			}
			catch(SoapException e)
			{
				Trace.WriteLine(e.Message);
				this.DoLogMessage("Web Exception: " + e.Message, true);
				return;
			}

			// check error code
			if(tickets[0].Equals("S_ERROR_INVALID_LOGIN"))
			{
				this.DoLogMessage("Login error.", true);
				return;
			}

			// check error code
			if(tickets[0].Equals("S_ERROR_NOTHING_FOUND"))
			{
				this.DoLogMessage("No tickets available.", true);
				return;
			}

			/* here we don't have to expect ExperimentType nodes to get deleted in HandleTicketList()
				   (only Jobs are considered) */
			ArrayList ticketList = new ArrayList(tickets);
			foreach(TreeNode node in this.treeView1.Nodes)
			{
				if(node is ExperimentType)
				{
					((ExperimentType)node).HandleTicketList(ticketList, this);
				}
			}

			// all tickets left are new and must be added to their according experiment type node
			foreach(string ticket in ticketList)
			{
				// get experiment of job
				string expType = null;
				try
				{
					expType = this.experimentWebService.GetExperimentTypeofTicket(ticket);
				}
				catch(SoapException e)
				{
					Trace.WriteLine(e.Message);
					this.DoLogMessage("Web Exception: " + e.Message, true);
					continue;
				}

				if((expType != null) && (!expType.Equals("")) && (!expType.Equals("S_ERROR_Ticket not found")))
				{
					ExperimentType foundExpType = null;

					foreach(ExperimentType type in this.treeView1.Nodes)
					{
						if(type.Name.Equals(expType))
						{
							foundExpType = type;
							break;
						}
					}

					if(foundExpType != null)
					{
						string eval = null;
						try
						{
							eval = this.ticketWebService.CheckTicket(ticket);
						}
						catch(SoapException e)
						{
							Trace.WriteLine(e.Message);
							this.DoLogMessage("Web Exception: " + e.Message, true);
							continue;
						}

						if(eval != null)
						{
							if(eval.Equals("S_ERROR_UNKNOWN_TICKET"))
							{
								this.DoLogMessage("Unknown ticket.", true);
								continue;
							}

							if(eval.Equals("S_ERROR_TICKET_EXPIRED"))
							{
								this.DoLogMessage("Ticket expired.", true);
								continue;
							}

							if(eval.Equals("S_OK"))
							{
								Job job = new Job(ticket, this);

								try
								{
									this.treeView1.Invoke(new AddNodeHandler(this.addNodeCallback), 
										new object[] { job, foundExpType.Nodes });

									if (ticket == LastSentJobTicket)
									{
										// set the last sent job highlight color
										this.LastSentJobNode = job;
										this.LastSentJobTicket = "";

										this.LastSentJobNode.BackColor = LastSentHighlightColor;

										this.LastSentJobNode.Parent.Expand();
										this.LastSentJobNode.ExpandAll();
									}
								}
								// this happens in Invoke when no handle is created for the treeview control
								catch(InvalidOperationException)
								{}
							}
						}
					}
					// a ticket for an experiment, that does not exist according to ListExperiments()
					else
					{
						try
						{
							ExperimentType experimentType = new ExperimentType(expType, false, this);
							this.treeView1.Invoke(new AddNodeHandler(this.addNodeCallback),
								new object[] { experimentType, this.treeView1.Nodes });

							Job job = new Job(ticket, this);
							this.treeView1.Invoke(new AddNodeHandler(this.addNodeCallback), 
								new object[] { job, experimentType.Nodes });

							if (ticket == LastSentJobTicket)
							{
								// set the last sent job highlight color
								this.LastSentJobNode = job;
								this.LastSentJobTicket = "";

								this.LastSentJobNode.BackColor = LastSentHighlightColor;

								this.LastSentJobNode.Parent.Expand();
								this.LastSentJobNode.ExpandAll();
							}
						}
						// this happens in Invoke when no handle is created for the treeview control
						catch(InvalidOperationException)
						{}
					}
				}
			}
		}

		//private int loopCount = 0;

		private void updateJobViewControl()
		{
			//this.DoLogMessage("update loop cycle: " + (++this.loopCount), true);

			this.updateExperiments();
			
			this.updateJobs();
		}

		private void updateThreadRun()
		{
			while (true)
			{
				try
				{
					try
					{
						try
						{
							this.updateJobViewControl();
							Thread.Sleep(this.config.RefreshInterval * 1000);
						}
						catch(System.Net.WebException)
						{
							DoLogMessage("Could not connect to DCL WebService. Will try again in " +
								this.config.RefreshInterval + " seconds.", true);
							Thread.Sleep(this.config.RefreshInterval * 1000);
						}
					}
					catch(ThreadInterruptedException)
					{
						// we only want to get interrupted to be able to receive ThreadAbortedExceptions
					}
				}
					// this exception also comes when the thread sleeps
				catch(ThreadAbortException tae)
				{
					if (tae.ExceptionState is AbortType)
					{
						if ((AbortType) tae.ExceptionState == AbortType.CONFIG_CHANGED)
						{
							// we don't want to get killed
							Thread.ResetAbort();

							try
							{
								// reset the treeview
								this.treeView1.Invoke(new ClearNodesHandler(this.clearNodesCallback),
									new object[] {this.treeView1.Nodes});
							}
							// this happens in Invoke when no handle is created for the treeview control
							catch(InvalidOperationException)
							{}
						}
						else if ((AbortType) tae.ExceptionState == AbortType.STOP ||
							(AbortType) tae.ExceptionState == AbortType.EXIT)
						{
							/* if we don't call reset abort, the ThreadAbortException 
							* is automatically re-thrown and the thread will therefore finish */
						}
					}
				}
			}
		}
		#endregion

		#region Public properties and events

		// these are public to access them from the Job class
		public string LastSentJobTicket;
		public Job LastSentJobNode;
		public ExperimentType LastSentExperimentNode;

		/// <summary>
		///		The webservice stub for the experiments.
		/// </summary>
		public DCLJobView.DCLExperimentService.DCLExperimentService ExpService
		{
			get { return this.experimentWebService; }
		}

		/// <summary>
		///		The webservice stub for tickets.
		/// </summary>
		public DCLJobView.DCLTicketServer.DCLTicketServer TicketService
		{
			get { return this.ticketWebService; }
		}

		/// <summary>
		///		The treeview component. A bit hackish, you could kill the treeview, if you wish.
		/// </summary>
		public TreeView Tree
		{
			get { return this.treeView1; }
		}

		/* Interesting note about events: the compiler for C# or VB.NET automatically creates a
		 * private variable xy for each event XYEventHandler with add_XY and remove_XY methods.
		 * See this link for more information about Events in .NET:
		 * http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpconprovidingeventfunctionality.asp */

		/// <summary>
		///		Called whenever the internal implementation has a log message.
		/// </summary>
		public event LogMessageEventHandler LogMessage;

		/// <summary>
		///		Called when the send job toolbar button or send job context menu item
		///		in the treeview has been clicked.
		/// </summary>
		public event EventHandler SendJobClicked;

		/// <summary>
		///		Called when the show result toolbar button or show result context menu item
		///		in the treeview has been clicked and the result is a string result.
		/// </summary>
		public event StringResultEventHandler ShowStringResultClicked;

		/// <summary>
		///		Called when the show result toolbar button or show result context menu item
		///		in the treeview has been clicked and the result is a compiler output result.
		/// </summary>
		public event CompilerResultEventHandler ShowCompilerResultClicked;

		/// <summary>
		///		Called when the show result toolbar button or show result context menu item
		///		in the treeview has been clicked and the result is a binary result.
		/// </summary>
		public event BinaryResultEventHandler ShowBinaryResultClicked;

		/// <summary>
		///		Called when the show web result toolbar button or show has been clicked.
		/// </summary>
		public event WebResultEventHandler ShowWebResultClicked;
		#endregion

		#region Public methods
		
		/// <summary>
		///		Reload the configuration from the config file. It resides in "%HOME%\.dcladdin\dcladdin.config".
		/// </summary>
		public void loadConfiguration()
		{
			// load configuration
			this.DoLogMessage("Loading configuration...", false);

			DirectoryInfo configDir = new DirectoryInfo(Environment.GetEnvironmentVariable("HOME") + "\\.dcladdin");
			if(!configDir.Exists) configDir.Create();

			FileInfo configFile = new FileInfo(configDir.FullName + "\\dcladdin.config");
			if(configFile.Exists)
			{
				this.config = (DCLJobView.Configuration)DCLJobView.ConfigurationManager.load(configFile);

				if(this.config != null)
					this.DoLogMessage("Configuration loaded.", false);
				else
				{
					this.DoLogMessage("Configuration can not be loaded.", false);
					this.config = new DCLJobView.Configuration(configFile.FullName);
				}
			}
			else
			{
				this.config = new DCLJobView.Configuration(configFile.FullName);
				this.DoLogMessage("Configuration can not be loaded.", false);
			}

			this.compilerOutputParser = new CompilerOutputParser(this.config.ErrorPatterns, this.config.CodeLinesOverhead);
			this.setServiceURLs();
		}

		/// <summary>
		///		Show the option dialog.
		/// </summary>
		public void showOptions()
		{
			if(!(this.config != null))
			{
				this.config = new Configuration(Environment.GetEnvironmentVariable("HOME") + "\\.dcladdin\\dcladdin.config");
			}
			OptionsForm OptDlg = new OptionsForm(this.config, this);
			OptDlg.ShowDialog();

			this.setServiceURLs();
		}

		/// <summary>
		///		Show a message box with the ticket of the selected job.
		/// </summary>
		public void showTicket()
		{
			if (this.treeView1.SelectedNode != null)
			{
				if (this.treeView1.SelectedNode is Job)
				{
					MessageBox.Show(((Job) this.treeView1.SelectedNode).Ticket, "Ticket ID");
				}
			}
		}

		/// <summary>
		///		Cancel the selected job (in the treeview).
		/// </summary>
		public void cancelJob()
		{
			if(this.treeView1.SelectedNode != null)
			{
				if(this.treeView1.SelectedNode is Job)
				{
					Job job = (Job)this.treeView1.SelectedNode;
					if(job.Status != Job.StatusKind.Canceled)
					{
						try
						{
							this.experimentWebService.CancelExperiment(job.Ticket);
							this.DoLogMessage("Job canceled with ticket " + job.Ticket + ".", false);
						}
						catch(SoapException e)
						{
							Trace.WriteLine(e.Message);
							this.DoLogMessage("Web Exception: " + e.Message, false);
						}
					}
				}
				else
					MessageBox.Show("Please select a job first.");
			}
		}

		/// <summary>
		///		Delete the selected job (in the treeview).
		/// </summary>
		public void deleteJob()
		{
			if(this.treeView1.SelectedNode != null)
			{
				if(this.treeView1.SelectedNode is Job)
				{
					if(MessageBox.Show("Are you sure you want to delete the selected ticket and its job?", "Confirmation",
						MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
					{

						try
						{
							string ticket = ((Job)this.treeView1.SelectedNode).Ticket;
							this.ticketWebService.FreeTicket(ticket);
							this.DoLogMessage("Job deleted with ticket " + ticket + ".", false);
							
							TreeNode job = this.treeView1.SelectedNode;
							this.treeView1.SelectedNode = this.treeView1.SelectedNode.Parent;
							job.Remove();
						}
						catch(SoapException e)
						{
							Trace.WriteLine(e.Message);
							this.DoLogMessage("Web Exception: " + e.Message, false);
						}
					}
				}
			}
		}

		/// <summary>
		///		Send a Job and choose the experiment type based on the selected node
		///		or otherwise on a dialog.
		/// </summary>
		/// <param name="jobCode">The code of the job as one string.</param>
		/// <param name="useSelectedNode">If true the currently selected experiment in the treeview will be used.</param>
		/// <returns>The ticket for the new job. Null if sending failed.</returns>
		public string sendJob(string jobCode, bool useSelectedNode)
		{
			if (this.treeView1.SelectedNode is ExperimentType && useSelectedNode)
			{
				if (((ExperimentType)this.treeView1.SelectedNode).Exists)
				{
					// that is good, we can take the name of the selected node
					return this.sendJob(((ExperimentType)this.treeView1.SelectedNode).Name, jobCode);
				}
			}
			else
			{
				string[] experiments = this.getExistingExperiments();

				if (experiments.Length == 0)
				{
					MessageBox.Show("No experiments found. Please open up the DCL Addin Tool Window and login to the DCL webservice in order to receive a list of experiments.");
				}
				else
				{
					// we must open a form to choose an experiment type
					ExperimentSelectionForm selectDlg = new ExperimentSelectionForm(this, experiments);

					// if the user presses cancel, we abort the process
					if (selectDlg.ShowDialog() == DialogResult.OK)
						if (selectDlg.SelectedExperiment != "")
							return this.sendJob(selectDlg.SelectedExperiment,jobCode);
				}
			}

			return null;
		}

		/// <summary>
		///		Send a Job.
		/// </summary>
		/// <param name="experimentType">The experiment to send the job to.</param>
		/// <param name="jobCode">The code of the job as one string.</param>
		/// <returns>The ticket for the new job. Null if sending failed.</returns>
		public string sendJob(string experimentType, string jobCode) // bool highlight, bool autoOpenResults
		{
			// get new ticket
			string ticket = null;
			try
			{
				ticket = this.ticketWebService.GetNewTicket(this.config.Login, this.config.Password);
			}
			catch(SoapException e)
			{
				Trace.WriteLine(e.Message);
				this.DoLogMessage("Web Exception: " + e.Message, false);
				return null;
			}

			// if we have a new ticket, start new job
			if(ticket != null)
			{
				try
				{
					this.experimentWebService.UseExperiment(ticket, experimentType, jobCode);
					this.DoLogMessage("New job started for experiment '" + experimentType + "' (ticket is '" + ticket + "').", false);

					// reset the last sent job color highlighting
					if (this.LastSentJobNode != null)
						this.LastSentJobNode.BackColor = SystemColors.Window;

					if (this.LastSentExperimentNode != null)
						this.LastSentExperimentNode.BackColor = SystemColors.Window;

					this.LastSentJobNode = null;
					this.LastSentJobTicket = ticket;

					this.LastSentExperimentNode = this.getExperimentTypeByName(experimentType);
					
					if (this.LastSentExperimentNode != null)
						this.LastSentExperimentNode.BackColor = LastSentHighlightColor;

					this.treeView1.SelectedNode = null;

					return ticket;
				}
				catch(SoapException e)
				{
					Trace.WriteLine(e.Message);
					this.DoLogMessage("Web Exception: " + e.Message, false);
				}
			}

			return null;
		}

		/// <summary>
		///		This method shouldn't be a public. Its only called by OptionsForm ???
		/// </summary>
		/// <param name="message"></param>
		/// <param name="synchWithGUI"></param>
		public void DoLogMessage(string message, bool synchWithGUI)
		{
			if (synchWithGUI)
			{
				if (this.treeView1 != null && this.treeView1.IsHandleCreated)
					this.treeView1.Invoke( new LogMessageCallbackHandler(this.LogMessageCallback),
						new object[] {message} );
			}
			else
			{
				LogMessageCallback(message);
			}
		}

		/// <summary>
		///		Call this from outside to start the whole automatic update process
		/// </summary>
		public void startUpdateProcess()
		{
			if(this.updateThread == null || !this.updateThread.IsAlive)
			{
				this.updateThread = new Thread(new ThreadStart(this.updateThreadRun));
				updateThread.Name = "UpdateThread";
				updateThread.Start();
			}
		}

		/// <summary>
		///		Call this from outside to stop the whole automatic update process
		/// </summary>
		public void stopUpdateProcess()
		{
			if(this.updateThread != null && this.updateThread.IsAlive)
			{
				this.updateThread.Interrupt();
				this.updateThread.Abort(AbortType.STOP);
			}
		}

		/// <summary>
		///		This should be called whenever the configuration changed -
		///		it will reinit the update loop of the thread.
		/// </summary>
		public void NetConfigurationChanged()
		{
			if (this.updateThread != null && this.updateThread.IsAlive)
			{
				this.updateThread.Interrupt();
				this.updateThread.Abort(AbortType.CONFIG_CHANGED);
			}
		}

		public void AdvancedConfigurationChanged()
		{
			foreach(TreeNode node in this.treeView1.Nodes)
			{
				foreach(Job job in node.Nodes)
				{
					job.updateIconsAndColors(this);
				}
			}
		}

		/// <summary>
		/// Indicates whether the update thread is running or not.
		/// </summary>
		public bool UpdateProcessRunning
		{
			get
			{
				return this.updateThread != null && this.updateThread.IsAlive;
			}
		}

		/// <summary>
		/// Retrieves all known experiments, existing and non-existing (older) ones.
		/// </summary>
		/// <returns>The names of the experiments</returns>
		public string[] getAllExperiments()
		{
			string[] experiments = new string[this.treeView1.Nodes.Count];
			for(int i=0; i<this.treeView1.Nodes.Count; i++)
			{
				experiments[i] = ((ExperimentType)this.treeView1.Nodes[i]).Name;
			}
			return experiments;
		}

		/// <summary>
		/// Retrieves all existing experiments for which jobs can be scheduled.
		/// </summary>
		/// <returns>The names of the experiments</returns>
		public string[] getExistingExperiments()
		{
			ArrayList experiments = new ArrayList(this.treeView1.Nodes.Count);

			foreach (TreeNode node in this.treeView1.Nodes)
			{
				if (((ExperimentType) node).Exists)
					experiments.Add(((ExperimentType) node).Name);
			}

			return (string[]) experiments.ToArray(typeof(string));
		}

		/// <summary>
		///		Returns the job node associated with this ticket.
		/// </summary>
		/// <param name="ticket">The unique identifier for jobs.</param>
		/// <returns>A job tree node.</returns>
		public Job getJob(string ticket)
		{
			if (ticket == null || ticket == "")
				return null;

			foreach (TreeNode expNode in this.treeView1.Nodes)
			{
				foreach (TreeNode jobNode in expNode.Nodes)
				{
					if (jobNode is Job)
					{
						if (((Job) jobNode).Ticket == ticket)
						{
							return (Job) jobNode;
						}
					}
				}
			}

			return null;
		}

		/// <summary>
		///		Returns the experiment type associated with this ticket.
		/// </summary>
		/// <param name="ticket">The unique identifier for jobs.</param>
		/// <returns>An experiment type tree node.</returns>
		public ExperimentType getExperimentType(string ticket)
		{
			if (ticket == null || ticket == "")
				return null;

			foreach (TreeNode expNode in this.treeView1.Nodes)
			{
				if (expNode is ExperimentType)
				{
					foreach (TreeNode jobNode in expNode.Nodes)
					{
						if (jobNode is Job)
						{
							if (((Job) jobNode).Ticket == ticket)
							{
								return (ExperimentType) expNode;
							}
						}
					}
				}
			}

			return null;
		}

		/// <summary>
		///		Retrieves a certain string result of a certain job.
		/// </summary>
		/// <param name="ticket">The unique identifier for jobs.</param>
		/// <param name="resultName">The name of the result.</param>
		/// <returns>The entire result as string. Null if the result is binary.</returns>
		public string getStringResult(string ticket, string resultName)
		{
			Result result = this.getResult(ticket, resultName);

			if (result != null)
			{
				if (result.IsBinary)
					return null;
				else
				{
					try
					{
						string resultString = "";

						resultString = this.experimentWebService.GetStringResult(ticket, result.Name);
						return resultString;
					}
					catch(SoapException e)
					{
						Trace.WriteLine(e.Message);
						this.DoLogMessage("Web Exception: " + e.Message, false);
						return null;
					}
				}
			}

			return null;
		}

		/// <summary>
		///		Retrieves a certain result of a certain job.
		/// </summary>
		/// <param name="ticket">The unique identifier for jobs.</param>
		/// <param name="resultName">The name of the result.</param>
		/// <returns>The result node.</returns>
		public Result getResult(string ticket, string name)
		{
			Job job = this.getJob(ticket);

			if (job != null)
			{
				foreach(Result result in job.Nodes)
				{
					return result;
				}
			}

			return null;
		}

		/// <summary>
		///		Returns the experiment type with that name.
		/// </summary>
		/// <param name="name">The unique identifier for experiments.</param>
		/// <returns>An experiment type tree node.</returns>
		public ExperimentType getExperimentTypeByName(string name)
		{
			if (name == null || name == "")
				return null;

			foreach (TreeNode expNode in this.treeView1.Nodes)
			{
				if (expNode is ExperimentType)
				{
					if (((ExperimentType) expNode).Name == name)
						return (ExperimentType) expNode;
				}
			}

			return null;
		}

		/// <summary>
		///		Do not call this method. It is a hack for something that should be inside this class
		///		but cannot be without large rewriting. This method is called by the Job class.
		/// </summary>
		/// <param name="job">The job of the according result</param>
		/// <param name="result">The result which is a "Compiler Output"</param>
		public void showCompilerResult(Job job, Result result)
		{
			if (job == null || result == null)
				return;

			if (result.Name == "Compiler Output")
			{
				try
				{
					string resultString = "";
					resultString = this.experimentWebService.GetStringResult(job.Ticket, result.Name);

					string experiment = "";
					if (job.Parent != null)
					{
						experiment = ((ExperimentType) job.Parent).Name;
					}
					else
					{
						/* Note: this webservice call is needed because in most cases the job node
						 * is not yet added to its parent experiment node when this method is called
						 * inside Job.Update() inside the Job constructor */
						experiment = null;
						try
						{
							experiment = this.experimentWebService.GetExperimentTypeofTicket(job.Ticket);
						}
						catch(SoapException e)
						{
							Trace.WriteLine(e.Message);
							this.DoLogMessage("Web Exception: " + e.Message, true);
						}
					}
					CompilerOutputItem[] items = compilerOutputParser.Parse(experiment, job.Ticket, resultString);
					this.OnShowCompilerResultClicked(new CompilerResultEvent(result.Name, result.ResultType, job.Ticket, items, resultString));
				}
				catch(SoapException e)
				{
					Trace.WriteLine(e.Message);
					this.DoLogMessage("Web Exception: " + e.Message, false);
					return;
				}
			}
		}

		/// <summary>
		/// Returns the color for a certain result code.
		/// </summary>
		/// <param name="resultCode">The result code string</param>
		/// <returns>A System.Drawing.Color object or Color.Black if nothing is found</returns>
		public Color ResultCodeToColor(string resultCode)
		{
			object obj = this.config.ResultCodeColors[resultCode];

			if (obj == null)
				return Color.Black;
			else
				return (Color) obj;
		}

		#endregion

		#region Internal Callbacks
		private void LogMessageCallback(string message)
		{
			this.OnLogMessage(new MessageEvent(message));
		}

		private void addNodeCallback(TreeNode node, TreeNodeCollection childs)
		{
			childs.Add(node);
		}

		private void clearNodesCallback(TreeNodeCollection nodes)
		{
			nodes.Clear();
		}

		#endregion
	}
}