using System;
using System.Collections;
using System.Windows.Forms;
using System.Web.Services.Protocols;

namespace DCLJobView.TreeModel
{
	/// <summary>
	///		Models a result of an experiment. 
	///		A result has a type and either a string or a binary content.
	/// </summary>
	public class Result : TreeNode
	{
		private string name;
		private string resultType;
		
		public Result(string name, string resultType)
		{
			this.name = name;
			this.resultType = resultType;

			// set node text
			this.Text = this.name + ": " + this.resultType;

			if(this.IsBinary)
			{
				this.ImageIndex = JobViewControl.BINARY_RESULT_IMAGE;
				this.SelectedImageIndex = JobViewControl.BINARY_RESULT_IMAGE;
			}
			else
			{
				if(this.resultType.Equals("Code"))
				{
					this.ImageIndex = JobViewControl.CODE_RESULT_IMAGE;
					this.SelectedImageIndex = JobViewControl.CODE_RESULT_IMAGE;
				}
				else if(this.resultType.Equals("Text"))
				{
					if(this.name.Equals("Compiler Output"))
					{
						this.ImageIndex = JobViewControl.COMPILER_RESULT_IMAGE;
						this.SelectedImageIndex = JobViewControl.COMPILER_RESULT_IMAGE;
					}
					else
					{
						this.ImageIndex = JobViewControl.TEXT_RESULT_IMAGE;
						this.SelectedImageIndex = JobViewControl.TEXT_RESULT_IMAGE;
					}
				}
			}
		}

		/// <summary>
		///		The identifying name of the result. 
		///		E.g. "Code", "Compiler Output".
		/// </summary>
		public string Name
		{
			get { return this.name; }
		}

		/// <summary>
		///		Types can be Text, State Flow, Video...
		/// </summary>
		public string ResultType
		{
			get { return this.resultType; }
		}

		public void HandleResultList(ArrayList resultList)
		{
			bool found = false;

			for(int i=0; i < resultList.Count; i++)
			{
				// check for our experiment type name
				if (resultList[i] != null && ((string) resultList[i]).Equals(this.name))
				{
					found = true;
					resultList.RemoveAt(i);

					break;
				}
			}

			// if we did not find ourself in the list, we must remove us from the tree
			if(!found) this.Remove();
		}

		/// <summary>
		/// Indicates if this is a binary result.
		/// </summary>
		public bool IsBinary
		{
			get
			{
				return !(this.resultType.Equals("Code") || this.resultType.Equals("Text"));
			}
		}

		/// <summary>
		/// Gets the job node for this result.
		/// </summary>
		public Job Job
		{
			get
			{
				if (this.Parent is Job)
					return (Job) this.Parent;

				else
					return null;
			}
		}
	}
}